/*
*   $Id: e_qdos.h,v 1.1 2002/03/08 22:35:19 darren Exp $
*
*   Copyright (c) 2002, Darren Hiebert
*
*   This source code is released for free distribution under the terms of the
*   GNU General Public License.
*
*   Configures ctags for QDOS environment.
*/
#ifndef E_QDOS_H
#define E_QDOS_H

#define HAVE_DIRENT_H 1
#define HAVE_STDLIB_H 1
#define HAVE_SYS_STAT_H 1
#define HAVE_SYS_TIMES_H 1
#define HAVE_SYS_TYPES_H 1
#define HAVE_TIME_H 1
#define HAVE_UNISTD_H 1
#define STDC_HEADERS 1
#define HAVE_CLOCK 1
#define HAVE_FGETPOS 1
#define HAVE_FTRUNCATE 1
#define HAVE_OPENDIR 1
#define HAVE_PUTENV 1
#define HAVE_REMOVE 1
#define HAVE_STRERROR 1
#define HAVE_STRSTR 1
#define HAVE_TIMES 1
#define HAVE_TRUNCATE 1
#define NON_CONST_PUTENV_PROTOTYPE 1

#endif
