wxDEFINE_EVENT(); wxDEFINE_EVENT(); wxDEFINE_EVENT(); static const wxString LexerTextDefaultXML =                                                                                                    ; class clCommandEvent; ColoursAndFontsManager::ColoursAndFontsManager()  : m_initialized() {} ColoursAndFontsManager::~ColoursAndFontsManager() {} ColoursAndFontsManager& ColoursAndFontsManager::Get() {}  struct ColoursAndFontsManagerLoaderHelper {  ColoursAndFontsManager* m_manager;  ColoursAndFontsManagerLoaderHelper()  : m_manager()  {}  void Load()  {  std::vector<wxXmlDocument*> userLexers;  wxArrayString files; struct stat buff; files.