//////////////////////////////////////////////////////////////////////
// This file was auto-generated by codelite's wxCrafter Plugin
// wxCrafter project file: envvar_table_dlg.wxcp
// Do not modify this file by hand!
//////////////////////////////////////////////////////////////////////

#ifndef CODELITE_PLUGIN_ENVVAR_TABLE_DLG_BASE_CLASSES_H
#define CODELITE_PLUGIN_ENVVAR_TABLE_DLG_BASE_CLASSES_H

#include <wx/settings.h>
#include <wx/xrc/xmlres.h>
#include <wx/xrc/xh_bmp.h>
#include <wx/dialog.h>
#include <wx/iconbndl.h>
#include <wx/artprov.h>
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/notebook.h>
#include <wx/panel.h>
#include <wx/imaglist.h>
#include <wx/stc/stc.h>
#include <wx/button.h>
#if wxVERSION_NUMBER >= 2900
#include <wx/persist.h>
#include <wx/persist/toplevel.h>
#include <wx/persist/bookctrl.h>
#include <wx/persist/treebook.h>
#endif

#ifdef WXC_FROM_DIP
#undef WXC_FROM_DIP
#endif
#if wxVERSION_NUMBER >= 3100
#define WXC_FROM_DIP(x) wxWindow::FromDIP(x, NULL)
#else
#define WXC_FROM_DIP(x) x
#endif

#include "codelite_exports.h"

class WXDLLIMPEXP_SDK EnvVarsTableDlgBase : public wxDialog
{
protected:
    wxStaticText* m_staticText1;
    wxStaticText* m_staticText2;
    wxNotebook* m_book;
    wxPanel* m_panel1;
    wxStyledTextCtrl* m_textCtrlDefault;
    wxButton* m_buttonNewSet;
    wxButton* m_buttonDeleteSet;
    wxButton* m_button5;
    wxButton* m_button16;
    wxButton* m_button14;

protected:
    virtual void OnClose(wxCloseEvent& event) { event.Skip(); }
    virtual void OnNewSet(wxCommandEvent& event) { event.Skip(); }
    virtual void OnDeleteSet(wxCommandEvent& event) { event.Skip(); }
    virtual void OnDeleteSetUI(wxUpdateUIEvent& event) { event.Skip(); }
    virtual void OnExport(wxCommandEvent& event) { event.Skip(); }
    virtual void OnCancel(wxCommandEvent& event) { event.Skip(); }
    virtual void OnButtonOk(wxCommandEvent& event) { event.Skip(); }

public:
    wxStaticText* GetStaticText1() { return m_staticText1; }
    wxStaticText* GetStaticText2() { return m_staticText2; }
    wxStyledTextCtrl* GetTextCtrlDefault() { return m_textCtrlDefault; }
    wxPanel* GetPanel1() { return m_panel1; }
    wxNotebook* GetBook() { return m_book; }
    wxButton* GetButtonNewSet() { return m_buttonNewSet; }
    wxButton* GetButtonDeleteSet() { return m_buttonDeleteSet; }
    wxButton* GetButton5() { return m_button5; }
    wxButton* GetButton16() { return m_button16; }
    wxButton* GetButton14() { return m_button14; }
    EnvVarsTableDlgBase(wxWindow* parent, wxWindowID id = wxID_ANY, const wxString& title = _("Environment Variables"), const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxSize(-1, -1), long style = wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER);
    virtual ~EnvVarsTableDlgBase();
};

#endif
