//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Copyright            : (C) 2015 Eran Ifrah
// File name            : qmakesettingsbasedlg.h
//
// -------------------------------------------------------------------------
// A
//              _____           _      _     _ _
//             /  __ \         | |    | |   (_) |
//             | /  \/ ___   __| | ___| |    _| |_ ___
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/
//              \____/\___/ \__,_|\___\_____/_|\__\___|
//
//                                                  F i l e
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// This file was auto-generated by codelite's wxCrafter Plugin
// wxCrafter project file: qmakesettingsbasedlg.wxcp
// Do not modify this file by hand!
//////////////////////////////////////////////////////////////////////

#ifndef CODELITE_QMAKEPLUGIN_QMAKESETTINGSBASEDLG_BASE_CLASSES_H
#define CODELITE_QMAKEPLUGIN_QMAKESETTINGSBASEDLG_BASE_CLASSES_H

#include <wx/settings.h>
#include <wx/xrc/xmlres.h>
#include <wx/xrc/xh_bmp.h>
#include <wx/dialog.h>
#include <wx/iconbndl.h>
#include <wx/artprov.h>
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/notebook.h>
#include <wx/statline.h>
#include <wx/button.h>
#include <wx/panel.h>
#include <wx/filepicker.h>
#include <wx/combobox.h>
#include <wx/arrstr.h>
#include <wx/textctrl.h>
#if wxVERSION_NUMBER >= 2900
#include <wx/persist.h>
#include <wx/persist/toplevel.h>
#include <wx/persist/bookctrl.h>
#include <wx/persist/treebook.h>
#endif

class QMakeSettingsBaseDlg : public wxDialog
{
protected:
    wxStaticText* m_staticText5;
    wxNotebook* m_notebook;
    wxStaticLine* m_staticline1;
    wxButton* m_buttonNew;
    wxButton* m_buttonCancel;
    wxButton* m_buttonOk;

protected:
    virtual void OnRightDown(wxMouseEvent& event) { event.Skip(); }
    virtual void OnNewQmakeSettings(wxCommandEvent& event) { event.Skip(); }
    virtual void OnOK(wxCommandEvent& event) { event.Skip(); }

public:
    wxStaticText* GetStaticText5() { return m_staticText5; }
    wxNotebook* GetNotebook() { return m_notebook; }
    wxStaticLine* GetStaticline1() { return m_staticline1; }
    wxButton* GetButtonNew() { return m_buttonNew; }
    wxButton* GetButtonCancel() { return m_buttonCancel; }
    wxButton* GetButtonOk() { return m_buttonOk; }
    QMakeSettingsBaseDlg(wxWindow* parent, wxWindowID id = wxID_ANY, const wxString& title = _("QMake Settings"), const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxSize(-1, -1), long style = wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER);
    virtual ~QMakeSettingsBaseDlg();
};


class QmakeSettingsTabBase : public wxPanel
{
protected:
    wxStaticText* m_staticText1;
    wxFilePickerCtrl* m_filePickerQmakeExec;
    wxStaticText* m_staticText2;
    wxComboBox* m_comboBoxQmakespec;
    wxStaticText* m_staticText4;
    wxTextCtrl* m_textCtrlQtdir;

protected:

public:
    wxStaticText* GetStaticText1() { return m_staticText1; }
    wxFilePickerCtrl* GetFilePickerQmakeExec() { return m_filePickerQmakeExec; }
    wxStaticText* GetStaticText2() { return m_staticText2; }
    wxComboBox* GetComboBoxQmakespec() { return m_comboBoxQmakespec; }
    wxStaticText* GetStaticText4() { return m_staticText4; }
    wxTextCtrl* GetTextCtrlQtdir() { return m_textCtrlQtdir; }
    QmakeSettingsTabBase(wxWindow* parent, wxWindowID id = wxID_ANY, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxSize(-1,-1), long style = wxTAB_TRAVERSAL);
    virtual ~QmakeSettingsTabBase();
};

#endif
