//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Copyright            : (C) 2015 Eran Ifrah
// File name            : importfilesdialog_new.h
//
// -------------------------------------------------------------------------
// A
//              _____           _      _     _ _
//             /  __ \         | |    | |   (_) |
//             | /  \/ ___   __| | ___| |    _| |_ ___
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/
//              \____/\___/ \__,_|\___\_____/_|\__\___|
//
//                                                  F i l e
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// This file was auto-generated by codelite's wxCrafter Plugin
// wxCrafter project file: importfilesdialog.wxcp
// Do not modify this file by hand!
//////////////////////////////////////////////////////////////////////

#ifndef CODELITE_LITEEDITOR_IMPORTFILESDIALOG_BASE_CLASSES_H
#define CODELITE_LITEEDITOR_IMPORTFILESDIALOG_BASE_CLASSES_H

#include <wx/settings.h>
#include <wx/xrc/xmlres.h>
#include <wx/xrc/xh_bmp.h>
#include <wx/dialog.h>
#include <wx/iconbndl.h>
#include <wx/artprov.h>
#include <wx/sizer.h>
#include <wx/bannerwindow.h>
#include <wx/filepicker.h>
#include <wx/dataview.h>
#include "foldermodel.h"
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/checkbox.h>
#include <wx/button.h>
#if wxVERSION_NUMBER >= 2900
#include <wx/persist.h>
#include <wx/persist/toplevel.h>
#include <wx/persist/bookctrl.h>
#include <wx/persist/treebook.h>
#endif

class ImportFilesDialogNewBase : public wxDialog
{
protected:
    wxBannerWindow* m_banner1;
    wxDirPickerCtrl* m_dirPicker;
    wxDataViewCtrl* m_dataview;
    wxObjectDataPtr<FolderModel> m_dataviewModel;

    wxStaticText* m_staticText1;
    wxTextCtrl* m_textCtrSpec;
    wxCheckBox* m_checkBoxFilesWOExt;
    wxStdDialogButtonSizer* m_stdBtnSizer7;
    wxButton* m_buttonOK;
    wxButton* m_buttonCancel;

protected:
    virtual void OnDirChanged(wxFileDirPickerEvent& event) { event.Skip(); }
    virtual void OnValueChanged(wxDataViewEvent& event) { event.Skip(); }
    virtual void OnItemExpanding(wxDataViewEvent& event) { event.Skip(); }

public:
    wxBannerWindow* GetBanner1() { return m_banner1; }
    wxDirPickerCtrl* GetDirPicker() { return m_dirPicker; }
    wxDataViewCtrl* GetDataview() { return m_dataview; }
    wxStaticText* GetStaticText1() { return m_staticText1; }
    wxTextCtrl* GetTextCtrSpec() { return m_textCtrSpec; }
    wxCheckBox* GetCheckBoxFilesWOExt() { return m_checkBoxFilesWOExt; }
    ImportFilesDialogNewBase(wxWindow* parent, wxWindowID id = wxID_ANY, const wxString& title = _("Import Files"), const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxSize(400,400), long style = wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER);
    virtual ~ImportFilesDialogNewBase();
};

#endif
