#ifndef INCLUDE_FINDER_H
#define INCLUDE_FINDER_H

#include <string>

extern int OptimizeScope(const std::string &inputScope, std::string &optimizedScope, int localsLine, std::string &localsScope);

#endif
