//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// copyright            : (C) 2008 by Eran Ifrah                            
// file name            : y.tab.h              
//                                                                          
// -------------------------------------------------------------------------
// A                                                                        
//              _____           _      _     _ _                            
//             /  __ \         | |    | |   (_) |                           
//             | /  \/ ___   __| | ___| |    _| |_ ___                      
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )                     
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/                     
//              \____/\___/ \__,_|\___\_____/_|\__\___|                     
//                                                                          
//                                                  F i l e                 
//                                                                          
//    This program is free software; you can redistribute it and/or modify  
//    it under the terms of the GNU General Public License as published by  
//    the Free Software Foundation; either version 2 of the License, or     
//    (at your option) any later version.                                   
//                                                                          
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
 /*#ifdef __cplusplus*/
namespace flex
{
/*#endif*/

#define AUTO 257
#define lexDOUBLE 258
#define lexINT 259
#define lexSTRUCT 260
#define BREAK 261
#define ELSE 262
#define lexLONG 263
#define SWITCH 264
#define CASE 265
#define lexENUM 266
#define REGISTER 267
#define TYPEDEF 268
#define lexCHAR 269
#define EXTERN 270
#define RETURN 271
#define UNION 272
#define lexCONST 273
#define lexFLOAT 274
#define lexSHORT 275
#define UNSIGNED 276
#define CONTINUE 277
#define FOR 278
#define SIGNED 279
#define lexVOID 280
#define lexDEFAULT 281
#define GOTO 282
#define SIZEOF 283
#define VOLATILE 284
#define DO 285
#define IF 286
#define STATIC 287
#define WHILE 288
#define NEW 289
#define lexDELETE 290
#define lexTHIS 291
#define lexOPERATOR 292
#define lexCLASS 293
#define lexNAMESPACE 294
#define lexPUBLIC 295
#define lexPROTECTED 296
#define lexPRIVATE 297
#define VIRTUAL 298
#define FRIEND 299
#define INLINE 300
#define OVERLOAD 301
#define IDENTIFIER 302
#define STRINGliteral 303
#define FLOATINGconstant 304
#define INTEGERconstant 305
#define CHARACTERconstant 306
#define OCTALconstant 307
#define HEXconstant 308
#define TYPEDEFname 309
#define lexARROW 310
#define ICR 311
#define DECR 312
#define LS 313
#define RS 314
#define LE 315
#define GE 316
#define EQ 317
#define NE 318
#define ANDAND 319
#define OROR 320
#define ELLIPSIS 321
#define CLCL 322
#define DOTstar 323
#define ARROWstar 324
#define MULTassign 325
#define DIVassign 326
#define MODassign 327
#define PLUSassign 328
#define MINUSassign 329
#define LSassign 330
#define RSassign 331
#define ANDassign 332
#define ERassign 333
#define ORassign 334
#define CComment 335
#define CPPComment 336
#define POUNDPOUND 337

//#ifdef __cplusplus
} // namespace flex
//#endif
