//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Copyright            : (C) 2015 Eran Ifrah
// File name            : new_php_workspace_dlg.h
//
// -------------------------------------------------------------------------
// A
//              _____           _      _     _ _
//             /  __ \         | |    | |   (_) |
//             | /  \/ ___   __| | ___| |    _| |_ ___
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/
//              \____/\___/ \__,_|\___\_____/_|\__\___|
//
//                                                  F i l e
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#ifndef __new_php_workspace_dlg__
#define __new_php_workspace_dlg__

/**
@file
Subclass of NewPHPWorkspaceBaseDlg, which is generated by wxFormBuilder.
*/

#include "php_ui.h"

//// end generated include

/** Implementing NewPHPWorkspaceBaseDlg */
class NewPHPWorkspaceDlg : public NewPHPWorkspaceBaseDlg
{
public:
    /** Constructor */
    NewPHPWorkspaceDlg(wxWindow* parent);
    virtual ~NewPHPWorkspaceDlg();
    //// end generated class members
    wxString GetWorkspacePath() const;
    bool IsCreateProject() const { return m_checkBoxAddProjectFromSources->IsChecked(); }

protected:
    virtual void OnBrowse(wxCommandEvent& event);
    virtual void OnCheckMakeSeparateDir(wxCommandEvent& event);
    virtual void OnOKUI(wxUpdateUIEvent& event);
    virtual void OnOK(wxCommandEvent& event);
    virtual void OnNameUpdated(wxCommandEvent& event);
};

#endif // __new_php_workspace_dlg__
