//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// copyright            : (C) 2014 Eran Ifrah
// file name            : view.cpp
//
// -------------------------------------------------------------------------
// A
//              _____           _      _     _ _
//             /  __ \         | |    | |   (_) |
//             | /  \/ ___   __| | ___| |    _| |_ ___
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/
//              \____/\___/ \__,_|\___\_____/_|\__\___|
//
//                                                  F i l e
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#include "view.h"

XS_IMPLEMENT_CLONABLE_CLASS(View,xsSerializable);

View::View():xsSerializable()
{
	initSerializable();
}
View::View(const View& obj):xsSerializable(obj)
{
	m_name = obj.m_name;
	m_parentName = obj.m_parentName;
	m_select = obj.m_select;
	initSerializable();
}

View::View(IDbAdapter* dbAdapter,const wxString& name, const wxString& parentName, const wxString& select):xsSerializable()
{
	m_name = name;
	m_parentName = parentName;
	m_select = select;
	m_pDbAdapter = dbAdapter;
	initSerializable();
}
View::~View()
{
}



void View::initSerializable()
{
	XS_SERIALIZE(m_name,wxT("m_name")); 
	XS_SERIALIZE(m_parentName,wxT("m_parentName")); 
	XS_SERIALIZE(m_select,wxT("m_select")); 
}

