//////////////////////////////////////////////////////////////////////
// This file was auto-generated by codelite's wxCrafter Plugin
// wxCrafter project file: CodeLiteDiff_UI.wxcp
// Do not modify this file by hand!
//////////////////////////////////////////////////////////////////////

#ifndef CODELITE_CODELITEDIFF_CODELITEDIFF_UI_BASE_CLASSES_H
#define CODELITE_CODELITEDIFF_CODELITEDIFF_UI_BASE_CLASSES_H

#include <wx/settings.h>
#include <wx/xrc/xmlres.h>
#include <wx/xrc/xh_bmp.h>
#include <wx/dialog.h>
#include <wx/iconbndl.h>
#include <wx/artprov.h>
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/button.h>
#include <wx/listbox.h>
#if wxVERSION_NUMBER >= 2900
#include <wx/persist.h>
#include <wx/persist/toplevel.h>
#include <wx/persist/bookctrl.h>
#include <wx/persist/treebook.h>
#endif

class NewFileComparisonBase : public wxDialog
{
protected:
    wxStaticText* m_staticText27;
    wxTextCtrl* m_textCtrlLeftFile;
    wxStaticText* m_staticText25;
    wxTextCtrl* m_textCtrlFileName;
    wxButton* m_button22;
    wxStaticText* m_staticText33;
    wxListBox* m_listBox16;
    wxStdDialogButtonSizer* m_stdBtnSizer8;
    wxButton* m_button10;
    wxButton* m_button12;

protected:
    virtual void OnBrowse(wxCommandEvent& event) { event.Skip(); }
    virtual void OnFileSelected(wxCommandEvent& event) { event.Skip(); }
    virtual void OnOKUI(wxUpdateUIEvent& event) { event.Skip(); }

public:
    wxStaticText* GetStaticText27() { return m_staticText27; }
    wxTextCtrl* GetTextCtrlLeftFile() { return m_textCtrlLeftFile; }
    wxStaticText* GetStaticText25() { return m_staticText25; }
    wxTextCtrl* GetTextCtrlFileName() { return m_textCtrlFileName; }
    wxButton* GetButton22() { return m_button22; }
    wxStaticText* GetStaticText33() { return m_staticText33; }
    wxListBox* GetListBox16() { return m_listBox16; }
    NewFileComparisonBase(wxWindow* parent, wxWindowID id = wxID_ANY, const wxString& title = _("Select File for Comparison"), const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxSize(-1,-1), long style = wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER);
    virtual ~NewFileComparisonBase();
};

#endif
