#include <wx/xrc/xmlres.h>

	int IDS_ERD_SAVE_SQL = XRCID("IDS_ERD_SAVE_SQL");
	int IDS_ERD_SAVE_BMP = XRCID("IDS_ERD_SAVE_BMP");
	int IDS_ERD_COMMIT = XRCID("IDS_ERD_COMMIT");
    int IDT_ERD_TOOL = XRCID("IDT_ERD_TOOL");
	int IDT_ERD_TABLE = XRCID("IDT_ERD_TABLE");
	int IDT_ERD_VIEW = XRCID("IDT_ERD_VIEW");
	int IDT_ERD_LINE = XRCID("IDT_ERD_LINE");
	int IDT_ERD_ALIGN_CIRCLE = XRCID("IDT_ERD_ALIGN_CIRCLE");
	int IDT_ERD_ALIGN_MESH = XRCID("IDT_ERD_ALIGN_MESH");
	int IDT_ERD_ALIGN_VTREE = XRCID("IDT_ERD_ALIGN_VTREE");
	int IDT_ERD_ALIGN_HTREE = XRCID("IDT_ERD_ALIGN_HTREE");
	int IDT_ERD_ZOOM100 = XRCID("IDT_ERD_ZOOM100");
	int IDT_ERD_ZOOMALL = XRCID("IDT_ERD_ZOOMALL");
	
	int IDR_POPUP_NEW_TABLE = XRCID("IDR_POPUP_NEW_TABLE");
	int IDR_POPUP_NEW_VIEW = XRCID("IDR_POPUP_NEW_VIEW");
	int IDR_POPUP_NEW_KEY = XRCID("IDR_POPUP_NEW_KEY");
	int IDR_POPUP_MI1 = XRCID("IDR_POPUP_MI1");
	int IDR_POPUP_MI2 = XRCID("IDR_POPUP_MI2");
	int IDR_POPUP_MI3 = XRCID("IDR_POPUP_MI3");
	int IDR_POPUP_COPY = XRCID("IDR_POPUP_COPY");
	int IDR_POPUP_CUT = XRCID("IDR_POPUP_CUT");
	int IDR_POPUP_PASTE = XRCID("IDR_POPUP_PASTE");
	int IDR_DBVIEWER_ADD_DATABASE = XRCID("IDR_DBVIEWER_ADD_DATABASE");
	int IDR_DBVIEWER_DROP_DATABASE = XRCID("IDR_DBVIEWER_DROP_DATABASE");
	int IDR_DBVIEWER_IMPORT_DATABASE = XRCID("IDR_DBVIEWER_IMPORT_DATABASE");
	int IDR_DBVIEWER_EXPORT_DATABASE = XRCID("IDR_DBVIEWER_EXPORT_DATABASE");
	int IDR_DBVIEWER_ADD_TABLE = XRCID("IDR_DBVIEWER_ADD_TABLE");
	int IDR_DBVIEWER_DROP_TABLE = XRCID("IDR_DBVIEWER_DROP_TABLE");
	int IDR_DBVIEWER_ERD_DB = XRCID("IDR_DBVIEWER_ERD_DB");
	int IDR_DBVIEWER_ERD_TABLE = XRCID("IDR_DBVIEWER_ERD_TABLE");
	int IDR_DBVIEWER_CLASS_DB = XRCID("IDR_DBVIEWER_CLASS_DB");
	int IDR_DBVIEWER_CLASS_TABLE = XRCID("IDR_DBVIEWER_CLASS_TABLE");
	int IDR_DBVIEWER_DUMP_DATABASE = XRCID("IDR_DBVIEWER_DUMP_DATABASE");
	
	int IDR_SQLCOMMAND_SELECT = XRCID("IDR_SQLCOMMAND_SELECT");
	int IDR_SQLCOMMAND_UPDATE = XRCID("IDR_SQLCOMMAND_UPDATE");
	int IDR_SQLCOMMAND_INSERT = XRCID("IDR_SQLCOMMAND_INSERT");
	int IDR_SQLCOMMAND_DELETE = XRCID("IDR_SQLCOMMAND_DELETE");
	