#ifndef __DATABASELAYER_DEF_H__
#define __DATABASELAYER_DEF_H__

#if defined(WXMAKINGLIB_DATABASELAYER)
  #define WXDLLIMPEXP_DATABASELAYER
#elif defined(WXMAKINGDLL_DATABASELAYER)
  #define WXDLLIMPEXP_DATABASELAYER WXEXPORT
#elif defined(WXUSINGDLL_DATABASELAYER)
  #define WXDLLIMPEXP_DATABASELAYER WXIMPORT
#else // not making nor using DLL
  #define WXDLLIMPEXP_DATABASELAYER
#endif

#endif // __DATABASELAYER_DEF_H__
