//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Copyright            : (C) 2015 Eran Ifrah
// File name            : external_tools.h
//
// -------------------------------------------------------------------------
// A
//              _____           _      _     _ _
//             /  __ \         | |    | |   (_) |
//             | /  \/ ___   __| | ___| |    _| |_ ___
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/
//              \____/\___/ \__,_|\___\_____/_|\__\___|
//
//                                                  F i l e
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// This file was auto-generated by codelite's wxCrafter Plugin
// wxCrafter project file: external_tools.wxcp
// Do not modify this file by hand!
//////////////////////////////////////////////////////////////////////

#ifndef CODELITE_FORMBUILDER_EXTERNAL_TOOLS_BASE_CLASSES_H
#define CODELITE_FORMBUILDER_EXTERNAL_TOOLS_BASE_CLASSES_H

#include <wx/settings.h>
#include <wx/xrc/xmlres.h>
#include <wx/xrc/xh_bmp.h>
#include <wx/dialog.h>
#include <wx/iconbndl.h>
#include <wx/artprov.h>
#include <wx/sizer.h>
#include <wx/listctrl.h>
#include <wx/button.h>
#include <wx/dataview.h>
#if wxVERSION_NUMBER >= 2900
#include <wx/persist.h>
#include <wx/persist/toplevel.h>
#include <wx/persist/bookctrl.h>
#include <wx/persist/treebook.h>
#endif

class ExternalToolBaseDlg : public wxDialog
{
protected:
    wxListCtrl* m_listCtrlTools;
    wxButton* m_buttonNewTool;
    wxButton* m_buttonEdit;
    wxButton* m_buttonDelete;
    wxStdDialogButtonSizer* m_stdBtnSizer2;
    wxButton* m_button4;
    wxButton* m_button6;

protected:
    virtual void OnItemActivated(wxListEvent& event) { event.Skip(); }
    virtual void OnItemDeSelected(wxListEvent& event) { event.Skip(); }
    virtual void OnItemSelected(wxListEvent& event) { event.Skip(); }
    virtual void OnButtonNew(wxCommandEvent& event) { event.Skip(); }
    virtual void OnButtonNewUI(wxUpdateUIEvent& event) { event.Skip(); }
    virtual void OnButtonEdit(wxCommandEvent& event) { event.Skip(); }
    virtual void OnButtonEditUI(wxUpdateUIEvent& event) { event.Skip(); }
    virtual void OnButtonDelete(wxCommandEvent& event) { event.Skip(); }
    virtual void OnButtonDeleteUI(wxUpdateUIEvent& event) { event.Skip(); }

public:
    wxListCtrl* GetListCtrlTools() { return m_listCtrlTools; }
    wxButton* GetButtonNewTool() { return m_buttonNewTool; }
    wxButton* GetButtonEdit() { return m_buttonEdit; }
    wxButton* GetButtonDelete() { return m_buttonDelete; }
    ExternalToolBaseDlg(wxWindow* parent, wxWindowID id = wxID_ANY, const wxString& title = _("External Tools"), const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxSize(-1,-1), long style = wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER);
    virtual ~ExternalToolBaseDlg();
};


class ExternalToolsManagerBase : public wxDialog
{
protected:
    wxDataViewListCtrl* m_dvListCtrlTasks;
    wxButton* m_buttonKill;
    wxButton* m_buttonKillAll;
    wxButton* m_buttonRefresh;
    wxButton* m_button33;

protected:
    virtual void OnKill(wxCommandEvent& event) { event.Skip(); }
    virtual void OnKillUI(wxUpdateUIEvent& event) { event.Skip(); }
    virtual void OnKillAll(wxCommandEvent& event) { event.Skip(); }
    virtual void OnKillAllUI(wxUpdateUIEvent& event) { event.Skip(); }
    virtual void OnRefresh(wxCommandEvent& event) { event.Skip(); }

public:
    wxDataViewListCtrl* GetDvListCtrlTasks() { return m_dvListCtrlTasks; }
    wxButton* GetButtonKill() { return m_buttonKill; }
    wxButton* GetButtonKillAll() { return m_buttonKillAll; }
    wxButton* GetButtonRefresh() { return m_buttonRefresh; }
    wxButton* GetButton33() { return m_button33; }
    ExternalToolsManagerBase(wxWindow* parent, wxWindowID id = wxID_ANY, const wxString& title = _("Tools"), const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxSize(500,300), long style = wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER);
    virtual ~ExternalToolsManagerBase();
};

#endif
