//////////////////////////////////////////////////////////////////////
// This file was auto-generated by codelite's wxCrafter Plugin
// wxCrafter project file: AccelTableBaseDlg.wxcp
// Do not modify this file by hand!
//////////////////////////////////////////////////////////////////////

#ifndef _CODELITE_LITEEDITOR_ACCELTABLEBASEDLG_BASE_CLASSES_H
#define _CODELITE_LITEEDITOR_ACCELTABLEBASEDLG_BASE_CLASSES_H

#include <wx/settings.h>
#include <wx/xrc/xmlres.h>
#include <wx/xrc/xh_bmp.h>
#include <wx/dialog.h>
#include <wx/iconbndl.h>
#include <wx/artprov.h>
#include <wx/sizer.h>
#include <wx/srchctrl.h>
#include <wx/dataview.h>
#include <wx/button.h>
#if wxVERSION_NUMBER >= 2900
#include <wx/persist.h>
#include <wx/persist/toplevel.h>
#include <wx/persist/bookctrl.h>
#include <wx/persist/treebook.h>
#endif

#ifdef WXC_FROM_DIP
#undef WXC_FROM_DIP
#endif
#if wxVERSION_NUMBER >= 3100
#define WXC_FROM_DIP(x) wxWindow::FromDIP(x, NULL)
#else
#define WXC_FROM_DIP(x) x
#endif


class AccelTableBaseDlg : public wxDialog
{
protected:
    wxSearchCtrl* m_textCtrlFilter;
    wxDataViewListCtrl* m_dvListCtrl;
    wxButton* m_buttonEdit;
    wxButton* m_buttonDefault;
    wxStdDialogButtonSizer* m_stdBtnSizer6;
    wxButton* m_button8;
    wxButton* m_button10;

protected:
    virtual void OnText(wxCommandEvent& event) { event.Skip(); }
    virtual void OnDVItemActivated(wxDataViewEvent& event) { event.Skip(); }
    virtual void OnEditButton(wxCommandEvent& event) { event.Skip(); }
    virtual void OnEditUI(wxUpdateUIEvent& event) { event.Skip(); }
    virtual void OnButtonDefaults(wxCommandEvent& event) { event.Skip(); }
    virtual void OnButtonOk(wxCommandEvent& event) { event.Skip(); }

public:
    wxSearchCtrl* GetTextCtrlFilter() { return m_textCtrlFilter; }
    wxDataViewListCtrl* GetDvListCtrl() { return m_dvListCtrl; }
    wxButton* GetButtonEdit() { return m_buttonEdit; }
    wxButton* GetButtonDefault() { return m_buttonDefault; }
    AccelTableBaseDlg(wxWindow* parent, wxWindowID id = wxID_ANY, const wxString& title = _("Keyboard Shortcuts"), const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxSize(-1,-1), long style = wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER);
    virtual ~AccelTableBaseDlg();
};

#endif
