#ifndef IDS_H_INCLUDED
#define IDS_H_INCLUDED


extern int IDS_ERD_SAVE_SQL;
extern int IDS_ERD_SAVE_BMP;
extern int IDS_ERD_COMMIT;
extern int IDT_ERD_TOOL;
extern int IDT_ERD_TABLE;
extern int IDT_ERD_VIEW;
extern int IDT_ERD_LINE;
extern int IDT_ERD_ALIGN_CIRCLE;
extern int IDT_ERD_ALIGN_MESH;
extern int IDT_ERD_ALIGN_VTREE;
extern int IDT_ERD_ALIGN_HTREE;
extern int IDT_ERD_ZOOM100;
extern int IDT_ERD_ZOOMALL;

extern int IDR_POPUP_NEW_TABLE;
extern int IDR_POPUP_NEW_VIEW;
extern int IDR_POPUP_NEW_KEY;
extern int IDR_POPUP_MI1;
extern int IDR_POPUP_MI2;
extern int IDR_POPUP_MI3;
extern int IDR_POPUP_COPY;
extern int IDR_POPUP_CUT;
extern int IDR_POPUP_PASTE;
extern int IDR_DBVIEWER_ADD_DATABASE;
extern int IDR_DBVIEWER_DROP_DATABASE;
extern int IDR_DBVIEWER_IMPORT_DATABASE;
extern int IDR_DBVIEWER_EXPORT_DATABASE;
extern int IDR_DBVIEWER_ADD_TABLE;
extern int IDR_DBVIEWER_DROP_TABLE;
extern int IDR_DBVIEWER_ERD_DB;
extern int IDR_DBVIEWER_ERD_TABLE;
extern int IDR_DBVIEWER_CLASS_DB;
extern int IDR_DBVIEWER_CLASS_TABLE;
extern int IDR_DBVIEWER_DUMP_DATABASE;

extern int IDR_SQLCOMMAND_SELECT;
extern int IDR_SQLCOMMAND_UPDATE;
extern int IDR_SQLCOMMAND_INSERT;
extern int IDR_SQLCOMMAND_DELETE;



#endif // IDS_H_INCLUDED