//////////////////////////////////////////////////////////////////////
// This file was auto-generated by codelite's wxCrafter Plugin
// wxCrafter project file: editor_options_comments_doxygen.wxcp
// Do not modify this file by hand!
//////////////////////////////////////////////////////////////////////

#ifndef _CODELITE_LITEEDITOR_EDITOR_OPTIONS_COMMENTS_DOXYGEN_BASE_CLASSES_H
#define _CODELITE_LITEEDITOR_EDITOR_OPTIONS_COMMENTS_DOXYGEN_BASE_CLASSES_H

#include <wx/settings.h>
#include <wx/xrc/xmlres.h>
#include <wx/xrc/xh_bmp.h>
#include <wx/panel.h>
#include <wx/artprov.h>
#include <wx/sizer.h>
#include <wx/propgrid/manager.h>
#include <wx/propgrid/property.h>
#include <wx/propgrid/advprops.h>
#if wxVERSION_NUMBER >= 2900
#include <wx/persist.h>
#include <wx/persist/toplevel.h>
#include <wx/persist/bookctrl.h>
#include <wx/persist/treebook.h>
#endif

#ifdef WXC_FROM_DIP
#undef WXC_FROM_DIP
#endif
#if wxVERSION_NUMBER >= 3100
#define WXC_FROM_DIP(x) wxWindow::FromDIP(x, NULL)
#else
#define WXC_FROM_DIP(x) x
#endif


class EditorSettingsCommentsDoxygenPanelBase : public wxPanel
{
protected:
    wxPropertyGridManager* m_pgMgrDoxy;
    wxPGProperty* m_pgProp10;
    wxPGProperty* m_pgPropAutoGen;
    wxPGProperty* m_pgPropCommentBlockPrefix;
    wxPGProperty* m_pgProp4;
    wxPGProperty* m_pgPropDoxyClassPrefix;
    wxPGProperty* m_pgPropDoxyFunctionPrefix;

protected:
    virtual void OnButtonClicked(wxCommandEvent& event) { event.Skip(); }

public:
    wxPropertyGridManager* GetPgMgrDoxy() { return m_pgMgrDoxy; }
    EditorSettingsCommentsDoxygenPanelBase(wxWindow* parent, wxWindowID id = wxID_ANY, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxSize(-1,-1), long style = wxTAB_TRAVERSAL);
    virtual ~EditorSettingsCommentsDoxygenPanelBase();
};

#endif
