//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Copyright            : (C) 2015 Eran Ifrah
// File name            : buildsettingstabbase.h
//
// -------------------------------------------------------------------------
// A
//              _____           _      _     _ _
//             /  __ \         | |    | |   (_) |
//             | /  \/ ___   __| | ___| |    _| |_ ___
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/
//              \____/\___/ \__,_|\___\_____/_|\__\___|
//
//                                                  F i l e
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// This file was auto-generated by codelite's wxCrafter Plugin
// wxCrafter project file: BuildTabSettings.wxcp
// Do not modify this file by hand!
//////////////////////////////////////////////////////////////////////

#ifndef CODELITE_LITEEDITOR_BUILDTABSETTINGS_BASE_CLASSES_H
#define CODELITE_LITEEDITOR_BUILDTABSETTINGS_BASE_CLASSES_H

#include <wx/settings.h>
#include <wx/xrc/xmlres.h>
#include <wx/xrc/xh_bmp.h>
#include <wx/panel.h>
#include <wx/artprov.h>
#include <wx/sizer.h>
#include <wx/propgrid/manager.h>
#include <wx/propgrid/property.h>
#include <wx/propgrid/advprops.h>
#if wxVERSION_NUMBER >= 2900
#include <wx/persist.h>
#include <wx/persist/toplevel.h>
#include <wx/persist/bookctrl.h>
#include <wx/persist/treebook.h>
#endif

class BuildTabSettingsBase : public wxPanel
{
protected:
    wxPropertyGridManager* m_pgMgr;
    wxPGProperty* CAT_COLOURS_AND_FONTS;
    wxPGProperty* m_pgPropFont;
    wxPGProperty* m_pgPropErrorColour;
    wxPGProperty* m_pgPropWarningColour;
    wxPGProperty* CAT_GENERAL;
    wxPGProperty* m_pgPropJumpWarnings;
    wxPGProperty* m_pgPropAutoHideBuildPane;
    wxPGProperty* m_pgPropAutoShowBuildPane;
    wxPGProperty* m_pgPropAutoScroll;
    wxPGProperty* CAT_MARKERS;
    wxPGProperty* m_pgPropUseMarkers;
    wxPGProperty* m_pgPropUseAnnotations;

protected:
    virtual void OnCustomButtonClicked(wxCommandEvent& event) { event.Skip(); }
    virtual void OnAppearanceChanged(wxPropertyGridEvent& event) { event.Skip(); }

public:
    wxPropertyGridManager* GetPgMgr() { return m_pgMgr; }
    BuildTabSettingsBase(wxWindow* parent, wxWindowID id = wxID_ANY, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxSize(-1,-1), long style = wxTAB_TRAVERSAL);
    virtual ~BuildTabSettingsBase();
};

#endif
