//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// copyright            : (C) 2008 by Eran Ifrah
// file name            : batchbuilddlg.h
//
// -------------------------------------------------------------------------
// A
//              _____           _      _     _ _
//             /  __ \         | |    | |   (_) |
//             | /  \/ ___   __| | ___| |    _| |_ ___
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/
//              \____/\___/ \__,_|\___\_____/_|\__\___|
//
//                                                  F i l e
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#ifndef __batchbuilddlg__
#define __batchbuilddlg__

/**
@file
Subclass of BatchBuildBaseDlg, which is generated by wxFormBuilder.
*/

#include "batchbuildbasedlg.h"
#include "shell_command.h"
#include <list>

/** Implementing BatchBuildBaseDlg */
class BatchBuildDlg : public BatchBuildBaseDlg
{
	void DoInitialize();
	void DoSaveBatchBuildOrder();

public:
	int m_cmd;

protected:
	// Handlers for BatchBuildBaseDlg events.
	void OnItemSelected( wxCommandEvent& event );
	void OnItemToggled( wxCommandEvent& event );
	void OnBuild( wxCommandEvent& event );
	void OnBuildUI( wxUpdateUIEvent& event );
	void OnClean( wxCommandEvent& event );
	void OnCleanUI( wxUpdateUIEvent& event );
	void OnCheckAll( wxCommandEvent& event );
	void OnUnCheckAll( wxCommandEvent& event );
	void OnMoveUp( wxCommandEvent& event );
	void OnMoveUpUI( wxUpdateUIEvent& event );
	void OnMoveDown( wxCommandEvent& event );
	void OnMoveDownUI( wxUpdateUIEvent& event );
	void OnClose( wxCommandEvent& event );

public:
	/** Constructor */
	BatchBuildDlg( wxWindow* parent );
	void GetBuildInfoList(std::list<QueueCommand> &buildInfoList);

};

#endif // __batchbuilddlg__
