//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Copyright            : (C) 2015 Eran Ifrah
// File name            : editorsettingsfoldingbase.h
//
// -------------------------------------------------------------------------
// A
//              _____           _      _     _ _
//             /  __ \         | |    | |   (_) |
//             | /  \/ ___   __| | ___| |    _| |_ ___
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/
//              \____/\___/ \__,_|\___\_____/_|\__\___|
//
//                                                  F i l e
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// This file was auto-generated by codelite's wxCrafter Plugin
// wxCrafter project file: editor_options_folding.wxcp
// Do not modify this file by hand!
//////////////////////////////////////////////////////////////////////

#ifndef CODELITE_LITEEDITOR_EDITOR_OPTIONS_FOLDING_BASE_CLASSES_H
#define CODELITE_LITEEDITOR_EDITOR_OPTIONS_FOLDING_BASE_CLASSES_H

#include <wx/settings.h>
#include <wx/xrc/xmlres.h>
#include <wx/xrc/xh_bmp.h>
#include <wx/panel.h>
#include <wx/artprov.h>
#include <wx/sizer.h>
#include <wx/checkbox.h>
#include <wx/statbox.h>
#include <wx/choice.h>
#include <wx/arrstr.h>
#if wxVERSION_NUMBER >= 2900
#include <wx/persist.h>
#include <wx/persist/toplevel.h>
#include <wx/persist/bookctrl.h>
#include <wx/persist/treebook.h>
#endif

class EditorSettingsFoldingBase : public wxPanel
{
protected:
    wxCheckBox* m_displayMargin;
    wxChoice* m_foldStyle;
    wxCheckBox* m_underlineFolded;
    wxCheckBox* m_foldElse;
    wxCheckBox* m_foldPreprocessors;
    wxCheckBox* m_foldCompact;
    wxCheckBox* m_checkBoxHighlightFolding;

protected:
    virtual void OnFoldingMarginUI(wxUpdateUIEvent& event) { event.Skip(); }

public:
    wxCheckBox* GetDisplayMargin() { return m_displayMargin; }
    wxChoice* GetFoldStyle() { return m_foldStyle; }
    wxCheckBox* GetUnderlineFolded() { return m_underlineFolded; }
    wxCheckBox* GetFoldElse() { return m_foldElse; }
    wxCheckBox* GetFoldPreprocessors() { return m_foldPreprocessors; }
    wxCheckBox* GetFoldCompact() { return m_foldCompact; }
    wxCheckBox* GetCheckBoxHighlightFolding() { return m_checkBoxHighlightFolding; }
    EditorSettingsFoldingBase(wxWindow* parent, wxWindowID id = wxID_ANY, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxSize(-1,-1), long style = wxTAB_TRAVERSAL);
    virtual ~EditorSettingsFoldingBase();
};

#endif
