//////////////////////////////////////////////////////////////////////
// This file was auto-generated by codelite's wxCrafter Plugin
// wxCrafter project file: wxcrafter.wxcp
// Do not modify this file by hand!
//////////////////////////////////////////////////////////////////////

#ifndef _CODELITE_OUTLINE_WXCRAFTER_BASE_CLASSES_H
#define _CODELITE_OUTLINE_WXCRAFTER_BASE_CLASSES_H

#include <wx/settings.h>
#include <wx/xrc/xmlres.h>
#include <wx/xrc/xh_bmp.h>
#include <wx/panel.h>
#include <wx/artprov.h>
#include <wx/sizer.h>
#include <wx/textctrl.h>
#include <wx/simplebook.h>
#include <wx/imaglist.h>
#include <wx/pen.h>
#include <wx/aui/auibar.h>
#include <map>
#include <wx/menu.h>
#include <wx/toolbar.h>
#include <wx/treectrl.h>
#include "PHPOutlineTree.h"
#if wxVERSION_NUMBER >= 2900
#include <wx/persist.h>
#include <wx/persist/toplevel.h>
#include <wx/persist/bookctrl.h>
#include <wx/persist/treebook.h>
#endif

#ifdef WXC_FROM_DIP
#undef WXC_FROM_DIP
#endif
#if wxVERSION_NUMBER >= 3100
#define WXC_FROM_DIP(x) wxWindow::FromDIP(x, NULL)
#else
#define WXC_FROM_DIP(x) x
#endif


class OutlineTabBaseClass : public wxPanel
{
public:
    enum {
        ID_TOOL_SORT_ALPHABETICALLY = 1001,
    };
protected:
    wxTextCtrl* m_textCtrlSearch;
    wxSimplebook* m_simpleBook;
    wxPanel* m_panelCxx;
    wxAuiToolBar* m_auibar;
    wxPanel* m_panelPhp;
    PHPOutlineTree* m_treeCtrlPhp;
    wxPanel* m_panelPlaceHolder;
    wxPanel* m_panelEmpty;

protected:
    virtual void OnSearchEnter(wxCommandEvent& event) { event.Skip(); }
    virtual void OnSearchSymbol(wxCommandEvent& event) { event.Skip(); }
    virtual void OnSortAlpha(wxCommandEvent& event) { event.Skip(); }
    virtual void OnSortAlphaUI(wxUpdateUIEvent& event) { event.Skip(); }
    virtual void OnPhpItemSelected(wxTreeEvent& event) { event.Skip(); }
    virtual void OnPhpItemActivated(wxTreeEvent& event) { event.Skip(); }

public:
    wxTextCtrl* GetTextCtrlSearch() { return m_textCtrlSearch; }
    wxAuiToolBar* GetAuibar() { return m_auibar; }
    wxPanel* GetPanelCxx() { return m_panelCxx; }
    PHPOutlineTree* GetTreeCtrlPhp() { return m_treeCtrlPhp; }
    wxPanel* GetPanelPhp() { return m_panelPhp; }
    wxPanel* GetPanelEmpty() { return m_panelEmpty; }
    wxPanel* GetPanelPlaceHolder() { return m_panelPlaceHolder; }
    wxSimplebook* GetSimpleBook() { return m_simpleBook; }
    OutlineTabBaseClass(wxWindow* parent, wxWindowID id = wxID_ANY, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxSize(-1,-1), long style = wxTAB_TRAVERSAL);
    virtual ~OutlineTabBaseClass();
};

#endif
