//////////////////////////////////////////////////////////////////////
// This file was auto-generated by codelite's wxCrafter Plugin
// wxCrafter project file: NewQtProj.wxcp
// Do not modify this file by hand!
//////////////////////////////////////////////////////////////////////

#ifndef CL_QMAKEPLUGIN_NEWQTPROJ_BASE_CLASSES_H
#define CL_QMAKEPLUGIN_NEWQTPROJ_BASE_CLASSES_H

#include <wx/settings.h>
#include <wx/xrc/xmlres.h>
#include <wx/xrc/xh_bmp.h>
#include <wx/dialog.h>
#include <wx/iconbndl.h>
#include <wx/artprov.h>
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/filepicker.h>
#include <wx/choice.h>
#include <wx/arrstr.h>
#include <wx/button.h>
#include <wx/statbox.h>
#include <wx/checkbox.h>
#if wxVERSION_NUMBER >= 2900
#include <wx/persist.h>
#include <wx/persist/toplevel.h>
#include <wx/persist/bookctrl.h>
#include <wx/persist/treebook.h>
#endif

class NewQtProjBaseDlg : public wxDialog
{
protected:
    wxStaticText* m_staticText5;
    wxTextCtrl* m_textCtrlProjName;
    wxStaticText* m_staticText7;
    wxDirPickerCtrl* m_dirPicker4;
    wxStaticText* m_staticText13;
    wxChoice* m_choiceProjKind;
    wxStaticText* m_staticText12;
    wxChoice* m_choiceQmake;
    wxButton* m_buttonSelectConfig;
    wxCheckBox* m_checkBoxUseSepDirectory;
    wxButton* m_buttonCancel;
    wxButton* m_buttonOk;

protected:
    virtual void OnNewQmakeSettings(wxCommandEvent& event) { event.Skip(); }
    virtual void OnOKUI(wxUpdateUIEvent& event) { event.Skip(); }

public:
    wxStaticText* GetStaticText5() { return m_staticText5; }
    wxTextCtrl* GetTextCtrlProjName() { return m_textCtrlProjName; }
    wxStaticText* GetStaticText7() { return m_staticText7; }
    wxDirPickerCtrl* GetDirPicker4() { return m_dirPicker4; }
    wxStaticText* GetStaticText13() { return m_staticText13; }
    wxChoice* GetChoiceProjKind() { return m_choiceProjKind; }
    wxStaticText* GetStaticText12() { return m_staticText12; }
    wxChoice* GetChoiceQmake() { return m_choiceQmake; }
    wxButton* GetButtonSelectConfig() { return m_buttonSelectConfig; }
    wxCheckBox* GetCheckBoxUseSepDirectory() { return m_checkBoxUseSepDirectory; }
    wxButton* GetButtonCancel() { return m_buttonCancel; }
    wxButton* GetButtonOk() { return m_buttonOk; }
    NewQtProjBaseDlg(wxWindow* parent, wxWindowID id = wxID_ANY, const wxString& title = _("New Qmake project"), const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxSize(-1, -1), long style = wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER);
    virtual ~NewQtProjBaseDlg();
};

#endif
