//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// copyright            : (C) 2008 by Eran Ifrah                            
// file name            : lex.yy.cpp              
//                                                                          
// -------------------------------------------------------------------------
// A                                                                        
//              _____           _      _     _ _                            
//             /  __ \         | |    | |   (_) |                           
//             | /  \/ ___   __| | ___| |    _| |_ ___                      
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )                     
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/                     
//              \____/\___/ \__,_|\___\_____/_|\__\___|                     
//                                                                          
//                                                  F i l e                 
//                                                                          
//    This program is free software; you can redistribute it and/or modify  
//    it under the terms of the GNU General Public License as published by  
//    the Free Software Foundation; either version 2 of the License, or     
//    (at your option) any later version.                                   
//                                                                          
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
 /* A lexical scanner generated by flex */

/* Scanner skeleton version:
 * $Header: /CVS/wxUS/wxXtudio/Apps/USClient/lex.yy.cpp,v 1.3 2005/12/13 07:53:23 eran Exp $
 */
 
/* cfront 1.2 defines "c_plusplus" instead of "__cplusplus" */
#ifdef c_plusplus
#ifndef __cplusplus
#define __cplusplus
#endif
#endif

#ifdef __cplusplus
#include "precompiled_header.h"
#include <stdlib.h>
#include "unistd.h"
#include <iostream>

#include "FlexLexer.h"
#endif

namespace flex
{
    
#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5

#ifdef __cplusplus

/* Use prototypes in function declarations. */
#define YY_USE_PROTOS

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else /* ! __cplusplus */

#if __STDC__

#define YY_USE_PROTOS
#define YY_USE_CONST

#endif /* __STDC__ */
#endif /* ! __cplusplus */

#ifdef __TURBOC__
 #pragma warn -rch
 #pragma warn -use
#include <io.h>
#include <stdlib.h>
#define YY_USE_CONST
#define YY_USE_PROTOS
#endif

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif


#ifdef YY_USE_PROTOS
#define YY_PROTO(proto) proto
#else
#define YY_PROTO(proto) ()
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN yy_start = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START ((yy_start - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE yyrestart( yyin )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#define YY_BUF_SIZE 16384

typedef struct yy_buffer_state *YY_BUFFER_STATE;

extern int yyleng; 

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

/* The funky do-while in the following #define is used to turn the definition
 * int a single C statement (which needs a semi-colon terminator).  This
 * avoids problems with code like:
 *
 *  if ( condition_holds )
 *  yyless( 5 );
 * else
 *  do_something_else();
 *
 * Prior to using the do-while the compiler would get upset at the
 * "else" because it interpreted the "if" statement as being all
 * done when it reached the ';' after the yyless() call.
 */

/* Return all but the first 'n' matched characters back to the input stream. */

#define yyless(n) \
	do \
	{ \
		/* Undo effects of setting up yytext. */ \
		*yy_cp = yy_hold_char; \
		YY_RESTORE_YY_MORE_OFFSET \
		yy_c_buf_p = yy_cp = yy_bp + n - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
	} \
	while ( 0 )

#define unput(c) yyunput( c, yytext_ptr )

/* The following is because we cannot portably get our hands on size_t
 * (without autoconf's help, which isn't available because we want
 * flex-generated scanners to compile on their own).
 */
typedef unsigned int yy_size_t;


struct yy_buffer_state
{
	std::istream* yy_input_file;

	char *yy_ch_buf;  /* input buffer */
	char *yy_buf_pos;  /* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;
#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via yyrestart()), so that the user can continue scanning by
	 * just pointing yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

};


/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 */
#define YY_CURRENT_BUFFER yy_current_buffer



static void *yy_flex_alloc YY_PROTO(( yy_size_t ));
static void *yy_flex_realloc YY_PROTO(( void *, yy_size_t ));
static void yy_flex_free YY_PROTO(( void * ));

#define yy_new_buffer yy_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
		if ( ! yy_current_buffer ) \
			yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE ); \
		yy_current_buffer->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
		if ( ! yy_current_buffer ) \
			yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE ); \
		yy_current_buffer->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (yy_current_buffer->yy_at_bol)


#define YY_USES_REJECT
typedef unsigned char YY_CHAR;
#define yytext_ptr yytext
#define YY_INTERACTIVE

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
	yytext_ptr = yy_bp; \
	yyleng = (int) (yy_cp - yy_bp); \
	yy_hold_char = *yy_cp; \
	*yy_cp = '\0'; \
	yy_c_buf_p = yy_cp;

#define YY_NUM_RULES 121
#define YY_END_OF_BUFFER 122
static yyconst short int yy_acclist[493] =
    { 0,
      122, 120, 121, 3, 120, 121, 5, 121, 4, 120,
      121, 84, 120, 121, 120, 121, 72, 120, 121, 86,
      120, 121, 79, 120, 121, 120, 121, 69, 120, 121,
      70, 120, 121, 80, 120, 121, 81, 120, 121, 71,
      120, 121, 82, 120, 121, 78, 120, 121, 85, 120,
      121, 64, 120, 121, 63, 120, 121, 92, 120, 121,
      93, 120, 121, 87, 120, 121, 94, 120, 121, 88,
      120, 121, 91, 120, 121, 62, 120, 121, 62, 120,
      121, 76, 120, 121, 77, 120, 121, 89, 120, 121,
      62, 120, 121, 62, 120, 121, 62, 120, 121, 62,

      120, 121, 62, 120, 121, 62, 120, 121, 62, 120,
      121, 62, 120, 121, 62, 120, 121, 62, 120, 121,
      62, 120, 121, 62, 120, 121, 62, 120, 121, 62,
      120, 121, 62, 120, 121, 62, 120, 121, 62, 120,
      121, 62, 120, 121, 74, 120, 121, 90, 120, 121,
      75, 120, 121, 83, 120, 121, 3, 5, 4, 106,
      68, 73, 111, 107, 116, 109, 99, 112, 100, 113,
      97, 95, 66, 1, 2, 110, 66, 64, 64, 64,
      63, 63, 63, 96, 101, 103, 105, 104, 102, 62,
      117, 62, 62, 62, 62, 62, 62, 62, 15, 62,

      62, 62, 62, 62, 62, 62, 62, 62, 26, 62,
      62, 62, 62, 62, 62, 62, 62, 62, 62, 62,
      62, 62, 62, 62, 62, 62, 62, 62, 62, 62,
      118, 108, 67, 98, 119, 66, 66, 64, 65, 63,
      114, 115, 62, 62, 62, 62, 62, 62, 62, 62,
      62, 62, 62, 62, 62, 62, 62, 62, 24, 62,
      62, 62, 62, 62, 62, 62, 30, 62, 62, 62,
      62, 54, 62, 62, 62, 62, 62, 62, 62, 62,
      62, 62, 62, 62, 62, 62, 62, 62, 62, 62,
      62, 62, 62, 62, 62, 62, 66, 65, 65, 6,

      62, 62, 8, 62, 9, 62, 62, 62, 62, 62,
      62, 62, 62, 17, 62, 18, 62, 62, 20, 62,
      62, 62, 62, 62, 25, 62, 62, 62, 62, 62,
      31, 62, 32, 62, 62, 62, 62, 62, 62, 62,
      62, 62, 62, 62, 62, 62, 62, 62, 62, 60,
      62, 62, 62, 62, 62, 62, 46, 62, 62, 62,
      65, 7, 62, 49, 62, 10, 62, 62, 62, 62,
      62, 62, 19, 62, 21, 62, 62, 23, 62, 62,
      27, 62, 62, 62, 62, 62, 62, 62, 62, 62,
      62, 62, 62, 62, 36, 62, 62, 62, 62, 62,

      62, 62, 43, 62, 44, 62, 62, 62, 62, 48,
      62, 62, 62, 13, 62, 51, 62, 16, 62, 22,
      62, 52, 62, 28, 62, 62, 53, 62, 62, 62,
      62, 33, 62, 62, 62, 59, 62, 62, 35, 62,
      37, 62, 38, 62, 39, 62, 40, 62, 41, 62,
      62, 62, 62, 62, 62, 12, 62, 14, 62, 29,
      62, 62, 62, 62, 58, 62, 62, 62, 42, 62,
      62, 61, 62, 62, 11, 62, 62, 55, 62, 56,
      62, 62, 34, 62, 45, 62, 47, 62, 50, 62,
      57, 62

    } ;

static yyconst short int yy_accept[365] =
    { 0,
      1, 1, 1, 2, 4, 7, 9, 12, 15, 17,
      20, 23, 26, 28, 31, 34, 37, 40, 43, 46,
      49, 52, 55, 58, 61, 64, 67, 70, 73, 76,
      79, 82, 85, 88, 91, 94, 97, 100, 103, 106,
      109, 112, 115, 118, 121, 124, 127, 130, 133, 136,
      139, 142, 145, 148, 151, 154, 157, 158, 159, 159,
      160, 161, 161, 162, 162, 163, 164, 165, 166, 166,
      166, 167, 168, 169, 170, 171, 172, 173, 173, 174,
      175, 176, 177, 178, 179, 179, 179, 180, 181, 181,
      182, 183, 184, 185, 186, 187, 188, 189, 190, 191,

      191, 191, 192, 193, 194, 195, 196, 197, 198, 199,
      201, 202, 203, 204, 205, 206, 207, 208, 209, 211,
      212, 213, 214, 215, 216, 217, 218, 219, 220, 221,
      222, 223, 224, 225, 226, 227, 228, 229, 230, 231,
      232, 233, 233, 233, 234, 234, 234, 235, 236, 236,
      237, 237, 238, 239, 240, 241, 242, 243, 244, 245,
      246, 247, 248, 249, 250, 251, 252, 253, 254, 255,
      256, 257, 258, 259, 261, 262, 263, 264, 265, 266,
      267, 269, 270, 271, 272, 274, 275, 276, 277, 278,
      279, 280, 281, 282, 283, 284, 285, 286, 287, 288,

      289, 290, 291, 292, 293, 294, 295, 296, 297, 297,
      297, 297, 297, 297, 298, 299, 300, 302, 303, 305,
      307, 308, 309, 310, 311, 312, 313, 314, 316, 318,
      319, 321, 322, 323, 324, 325, 327, 328, 329, 330,
      331, 333, 335, 336, 337, 338, 339, 340, 341, 342,
      343, 344, 345, 346, 347, 348, 349, 350, 352, 353,
      354, 355, 356, 357, 359, 360, 361, 361, 361, 361,
      361, 362, 364, 366, 368, 369, 370, 371, 372, 373,
      375, 377, 378, 380, 381, 383, 384, 385, 386, 387,
      388, 389, 390, 391, 392, 393, 394, 395, 397, 398,

      399, 400, 401, 402, 403, 405, 407, 408, 409, 410,
      412, 413, 414, 416, 418, 420, 422, 424, 426, 427,
      429, 430, 431, 432, 434, 435, 436, 438, 439, 441,
      443, 445, 447, 449, 451, 452, 453, 454, 455, 456,
      458, 460, 462, 463, 464, 465, 467, 468, 469, 471,
      472, 474, 475, 477, 478, 480, 482, 483, 485, 487,
      489, 491, 493, 493
    } ;

static yyconst int yy_ec[256] =
    { 0,
      1, 1, 1, 1, 1, 1, 1, 1, 2, 3,
      4, 5, 6, 1, 1, 1, 1, 1, 1, 1,
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
      1, 7, 8, 9, 10, 1, 11, 12, 13, 14,
      15, 16, 17, 18, 19, 20, 21, 22, 23, 23,
      23, 23, 23, 23, 23, 24, 24, 25, 26, 27,
      28, 29, 30, 1, 31, 31, 31, 31, 32, 33,
      34, 34, 34, 34, 34, 35, 34, 34, 34, 34,
      34, 34, 34, 34, 36, 34, 34, 37, 34, 34,
      38, 39, 40, 41, 34, 1, 42, 43, 44, 45,

      46, 47, 48, 49, 50, 34, 51, 52, 53, 54,
      55, 56, 34, 57, 58, 59, 60, 61, 62, 63,
      64, 65, 66, 67, 68, 69, 1, 1, 1, 1,
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,

      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
      1, 1, 1, 1, 1
    } ;

static yyconst int yy_meta[70] =
    { 0,
      1, 1, 2, 1, 1, 1, 1, 1, 3, 1,
      1, 1, 4, 1, 1, 1, 1, 1, 1, 1,
      1, 5, 5, 6, 1, 1, 1, 1, 1, 3,
      6, 6, 6, 7, 7, 7, 7, 1, 3, 1,
      1, 5, 5, 6, 6, 6, 5, 7, 7, 7,
      7, 7, 7, 8, 7, 7, 8, 7, 8, 7,
      8, 7, 8, 7, 7, 1, 1, 1, 1
    } ;

static yyconst short int yy_base[372] =
    { 0,
      0, 0, 570, 571, 68, 571, 74, 541, 73, 558,
      539, 71, 527, 571, 571, 537, 67, 571, 68, 69,
      82, 93, 100, 539, 571, 73, 535, 76, 571, 0,
      77, 571, 571, 534, 501, 503, 85, 56, 105, 86,
      504, 60, 71, 102, 86, 49, 512, 105, 69, 503,
      108, 507, 571, 103, 571, 571, 169, 571, 175, 181,
      571, 110, 571, 128, 571, 571, 571, 571, 153, 167,
      571, 571, 571, 571, 571, 539, 571, 534, 171, 571,
      571, 571, 175, 209, 192, 229, 160, 167, 0, 236,
      165, 174, 571, 525, 571, 571, 571, 524, 0, 196,

      512, 571, 491, 503, 490, 505, 504, 491, 187, 484,
      111, 168, 486, 483, 486, 483, 489, 479, 195, 198,
      483, 482, 482, 472, 487, 486, 212, 488, 152, 475,
      199, 221, 479, 478, 471, 225, 469, 224, 475, 571,
      571, 264, 0, 571, 267, 0, 571, 571, 275, 571,
      278, 285, 571, 269, 571, 571, 571, 469, 481, 476,
      464, 462, 178, 235, 473, 475, 470, 470, 465, 461,
      458, 466, 469, 0, 464, 454, 462, 462, 454, 455,
      0, 458, 455, 456, 0, 444, 443, 451, 437, 438,
      444, 445, 434, 436, 438, 445, 431, 429, 429, 429,

      440, 439, 429, 433, 423, 436, 438, 427, 301, 332,
      303, 367, 289, 335, 255, 279, 0, 427, 0, 0,
      419, 417, 425, 414, 419, 413, 419, 0, 0, 423,
      0, 412, 411, 408, 412, 0, 418, 418, 403, 408,
      0, 0, 403, 418, 407, 405, 415, 410, 405, 396,
      396, 386, 398, 384, 388, 382, 380, 0, 371, 361,
      353, 357, 344, 0, 344, 356, 256, 0, 280, 0,
      571, 0, 0, 0, 347, 345, 350, 349, 348, 0,
      0, 339, 0, 347, 0, 341, 341, 339, 328, 324,
      326, 331, 313, 325, 323, 307, 308, 0, 316, 313,

      309, 293, 302, 304, 0, 0, 295, 306, 297, 0,
      286, 286, 299, 0, 0, 0, 0, 0, 297, 0,
      297, 283, 294, 0, 289, 275, 0, 287, 0, 0,
      0, 0, 0, 0, 283, 282, 275, 270, 271, 0,
      0, 0, 240, 224, 234, 0, 220, 186, 0, 120,
      0, 62, 0, 48, 0, 0, 43, 0, 0, 0,
      0, 0, 571, 414, 422, 426, 432, 438, 442, 444,
      446
    } ;

static yyconst short int yy_def[372] =
    { 0,
      363, 1, 363, 363, 363, 363, 363, 363, 364, 363,
      363, 363, 365, 363, 363, 363, 363, 363, 363, 363,
      363, 363, 363, 363, 363, 363, 363, 363, 363, 366,
      366, 363, 363, 363, 366, 366, 366, 366, 366, 366,
      366, 366, 366, 366, 366, 366, 366, 366, 366, 366,
      366, 366, 363, 363, 363, 363, 363, 363, 363, 363,
      363, 364, 363, 367, 363, 363, 363, 363, 365, 368,
      363, 363, 363, 363, 363, 363, 363, 363, 363, 363,
      363, 363, 363, 363, 363, 363, 363, 363, 369, 363,
      363, 363, 363, 363, 363, 363, 363, 363, 366, 364,

      365, 363, 366, 366, 366, 366, 366, 366, 366, 366,
      366, 366, 366, 366, 366, 366, 366, 366, 366, 366,
      366, 366, 366, 366, 366, 366, 366, 366, 366, 366,
      366, 366, 366, 366, 366, 366, 366, 366, 366, 363,
      363, 364, 370, 363, 365, 371, 363, 363, 363, 363,
      363, 363, 363, 369, 363, 363, 363, 366, 366, 366,
      366, 366, 366, 366, 366, 366, 366, 366, 366, 366,
      366, 366, 366, 366, 366, 366, 366, 366, 366, 366,
      366, 366, 366, 366, 366, 366, 366, 366, 366, 366,
      366, 366, 366, 366, 366, 366, 366, 366, 366, 366,

      366, 366, 366, 366, 366, 366, 366, 366, 364, 364,
      365, 365, 363, 363, 363, 363, 366, 366, 366, 366,
      366, 366, 366, 366, 366, 366, 366, 366, 366, 366,
      366, 366, 366, 366, 366, 366, 366, 366, 366, 366,
      366, 366, 366, 366, 366, 366, 366, 366, 366, 366,
      366, 366, 366, 366, 366, 366, 366, 366, 366, 366,
      366, 366, 366, 366, 366, 366, 364, 210, 365, 212,
      363, 366, 366, 366, 366, 366, 366, 366, 366, 366,
      366, 366, 366, 366, 366, 366, 366, 366, 366, 366,
      366, 366, 366, 366, 366, 366, 366, 366, 366, 366,

      366, 366, 366, 366, 366, 366, 366, 366, 366, 366,
      366, 366, 366, 366, 366, 366, 366, 366, 366, 366,
      366, 366, 366, 366, 366, 366, 366, 366, 366, 366,
      366, 366, 366, 366, 366, 366, 366, 366, 366, 366,
      366, 366, 366, 366, 366, 366, 366, 366, 366, 366,
      366, 366, 366, 366, 366, 366, 366, 366, 366, 366,
      366, 366, 0, 363, 363, 363, 363, 363, 363, 363,
      363
    } ;

static yyconst short int yy_nxt[641] =
    { 0,
      4, 5, 6, 7, 7, 7, 5, 8, 9, 10,
      11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
      21, 22, 23, 23, 24, 25, 26, 27, 28, 29,
      30, 30, 30, 30, 31, 30, 30, 32, 4, 33,
      34, 35, 36, 37, 38, 39, 40, 41, 30, 42,
      30, 43, 30, 44, 45, 46, 47, 48, 49, 50,
      51, 52, 30, 30, 30, 53, 54, 55, 56, 57,
      58, 59, 59, 59, 57, 59, 58, 60, 60, 60,
      59, 63, 67, 72, 77, 100, 74, 362, 78, 101,
      79, 79, 79, 361, 73, 75, 76, 80, 68, 94,

      95, 109, 81, 97, 98, 127, 119, 360, 128, 82,
      110, 64, 83, 120, 84, 84, 85, 134, 63, 83,
      121, 90, 90, 90, 86, 122, 105, 87, 88, 89,
      140, 86, 135, 106, 91, 92, 107, 115, 86, 108,
      116, 125, 117, 123, 87, 86, 126, 124, 64, 142,
      142, 91, 88, 130, 131, 89, 111, 137, 112, 92,
      167, 113, 138, 132, 359, 144, 133, 114, 168, 141,
      57, 58, 59, 59, 59, 57, 59, 58, 59, 59,
      59, 59, 59, 58, 60, 60, 60, 59, 145, 145,
      143, 70, 79, 79, 79, 153, 79, 79, 79, 192,

      155, 153, 149, 150, 63, 150, 149, 150, 155, 150,
      193, 83, 169, 85, 85, 85, 149, 150, 153, 153,
      149, 150, 150, 86, 155, 155, 150, 170, 83, 146,
      84, 84, 85, 164, 64, 222, 223, 86, 165, 177,
      86, 179, 358, 87, 88, 151, 195, 151, 178, 180,
      152, 152, 152, 188, 86, 83, 181, 90, 90, 90,
      87, 189, 197, 196, 63, 357, 190, 86, 88, 202,
      91, 92, 63, 206, 203, 207, 224, 198, 356, 144,
      355, 86, 204, 354, 225, 209, 209, 91, 211, 211,
      271, 213, 144, 213, 64, 92, 214, 214, 214, 152,

      152, 152, 64, 215, 216, 70, 152, 152, 152, 63,
      214, 214, 214, 271, 271, 144, 353, 150, 70, 150,
      215, 352, 267, 267, 269, 269, 351, 350, 216, 349,
      271, 150, 348, 347, 346, 345, 150, 344, 343, 64,
      63, 70, 342, 341, 340, 339, 338, 337, 336, 335,
      334, 333, 332, 268, 268, 268, 214, 214, 214, 331,
      330, 329, 268, 268, 268, 328, 327, 150, 326, 150,
      64, 325, 324, 268, 268, 268, 268, 268, 268, 144,
      323, 150, 322, 321, 320, 319, 150, 318, 270, 270,
      270, 317, 316, 315, 314, 313, 312, 270, 270, 270,

      311, 310, 309, 308, 307, 70, 306, 305, 270, 270,
      270, 270, 270, 270, 62, 304, 62, 62, 62, 62,
      62, 62, 69, 303, 69, 302, 69, 69, 69, 69,
      99, 99, 99, 99, 62, 62, 62, 301, 300, 62,
      69, 69, 69, 299, 298, 69, 154, 154, 210, 210,
      212, 212, 297, 296, 295, 294, 293, 292, 291, 290,
      289, 288, 287, 286, 285, 284, 283, 282, 281, 280,
      279, 278, 277, 276, 275, 274, 273, 272, 266, 265,
      264, 263, 262, 261, 260, 259, 258, 257, 256, 255,
      254, 253, 252, 251, 250, 249, 248, 247, 246, 245,

      244, 243, 242, 241, 240, 239, 238, 237, 236, 235,
      234, 233, 232, 231, 230, 229, 228, 227, 226, 221,
      220, 219, 218, 217, 208, 205, 201, 200, 199, 194,
      191, 187, 186, 185, 184, 183, 182, 176, 175, 174,
      173, 172, 171, 166, 163, 162, 161, 160, 159, 158,
      70, 157, 156, 148, 147, 139, 136, 129, 118, 104,
      103, 102, 96, 93, 71, 70, 66, 65, 61, 363,
      3, 363, 363, 363, 363, 363, 363, 363, 363, 363,
      363, 363, 363, 363, 363, 363, 363, 363, 363, 363,
      363, 363, 363, 363, 363, 363, 363, 363, 363, 363,

      363, 363, 363, 363, 363, 363, 363, 363, 363, 363,
      363, 363, 363, 363, 363, 363, 363, 363, 363, 363,
      363, 363, 363, 363, 363, 363, 363, 363, 363, 363,
      363, 363, 363, 363, 363, 363, 363, 363, 363, 363
    } ;

static yyconst short int yy_chk[641] =
    { 0,
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
      1, 1, 1, 1, 1, 1, 1, 1, 1, 5,
      5, 5, 5, 5, 5, 7, 7, 7, 7, 7,
      7, 9, 12, 17, 20, 31, 19, 357, 20, 31,
      20, 20, 20, 354, 17, 19, 19, 21, 12, 26,

      26, 38, 21, 28, 28, 46, 42, 352, 46, 21,
      38, 9, 22, 42, 22, 22, 22, 49, 62, 23,
      43, 23, 23, 23, 22, 43, 37, 22, 22, 22,
      54, 23, 49, 37, 23, 23, 37, 40, 22, 37,
      40, 45, 40, 44, 22, 23, 45, 44, 62, 64,
      64, 23, 22, 48, 48, 22, 39, 51, 39, 23,
      111, 39, 51, 48, 350, 69, 48, 39, 111, 54,
      57, 57, 57, 57, 57, 57, 59, 59, 59, 59,
      59, 59, 60, 60, 60, 60, 60, 60, 70, 70,
      64, 69, 79, 79, 79, 87, 83, 83, 83, 129,

      91, 88, 79, 79, 100, 79, 83, 83, 92, 83,
      129, 85, 112, 85, 85, 85, 79, 79, 88, 87,
      83, 83, 79, 85, 91, 92, 83, 112, 84, 70,
      84, 84, 84, 109, 100, 163, 163, 85, 109, 119,
      84, 120, 348, 84, 84, 86, 131, 86, 119, 120,
      86, 86, 86, 127, 84, 90, 120, 90, 90, 90,
      84, 127, 132, 131, 267, 347, 127, 90, 84, 136,
      90, 90, 142, 138, 136, 138, 164, 132, 345, 145,
      344, 90, 136, 343, 164, 142, 142, 90, 145, 145,
      215, 149, 269, 149, 267, 90, 149, 149, 149, 151,

      151, 151, 142, 154, 154, 145, 152, 152, 152, 209,
      213, 213, 213, 216, 215, 211, 339, 152, 269, 152,
      154, 338, 209, 209, 211, 211, 337, 336, 154, 335,
      216, 152, 328, 326, 325, 323, 152, 322, 321, 209,
      210, 211, 319, 313, 312, 311, 309, 308, 307, 304,
      303, 302, 301, 210, 210, 210, 214, 214, 214, 300,
      299, 297, 210, 210, 210, 296, 295, 214, 294, 214,
      210, 293, 292, 210, 210, 210, 210, 210, 210, 212,
      291, 214, 290, 289, 288, 287, 214, 286, 212, 212,
      212, 284, 282, 279, 278, 277, 276, 212, 212, 212,

      275, 266, 265, 263, 262, 212, 261, 260, 212, 212,
      212, 212, 212, 212, 364, 259, 364, 364, 364, 364,
      364, 364, 365, 257, 365, 256, 365, 365, 365, 365,
      366, 366, 366, 366, 367, 367, 367, 255, 254, 367,
      368, 368, 368, 253, 252, 368, 369, 369, 370, 370,
      371, 371, 251, 250, 249, 248, 247, 246, 245, 244,
      243, 240, 239, 238, 237, 235, 234, 233, 232, 230,
      227, 226, 225, 224, 223, 222, 221, 218, 208, 207,
      206, 205, 204, 203, 202, 201, 200, 199, 198, 197,
      196, 195, 194, 193, 192, 191, 190, 189, 188, 187,

      186, 184, 183, 182, 180, 179, 178, 177, 176, 175,
      173, 172, 171, 170, 169, 168, 167, 166, 165, 162,
      161, 160, 159, 158, 139, 137, 135, 134, 133, 130,
      128, 126, 125, 124, 123, 122, 121, 118, 117, 116,
      115, 114, 113, 110, 108, 107, 106, 105, 104, 103,
      101, 98, 94, 78, 76, 52, 50, 47, 41, 36,
      35, 34, 27, 24, 16, 13, 11, 10, 8, 3,
      363, 363, 363, 363, 363, 363, 363, 363, 363, 363,
      363, 363, 363, 363, 363, 363, 363, 363, 363, 363,
      363, 363, 363, 363, 363, 363, 363, 363, 363, 363,

      363, 363, 363, 363, 363, 363, 363, 363, 363, 363,
      363, 363, 363, 363, 363, 363, 363, 363, 363, 363,
      363, 363, 363, 363, 363, 363, 363, 363, 363, 363,
      363, 363, 363, 363, 363, 363, 363, 363, 363, 363
    } ;

#define REJECT \
	{ \
		*yy_cp = yy_hold_char; /* undo effects of setting up yytext */ \
		yy_cp = yy_full_match; /* restore poss. backed-over text */ \
		++yy_lp; \
		goto find_rule; \
	}
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
#define INITIAL 0

/* Copyright (C) 1989-1991 James A. Roskind, All rights reserved.
This lexer description was written by James A.  Roskind.  Copying
of  this  file, as a whole, is permitted providing this notice is
intact  and  applicable   in   all   complete   copies.    Direct
translations  as a whole to other lexer generator input languages
(or lexical description languages)  is  permitted  provided  that
this  notice  is  intact and applicable in all such copies, along
with a disclaimer that  the  contents  are  a  translation.   The
reproduction  of derived files or text, such as modified versions
of this file, or the output of scanner generators, is  permitted,
provided   the  resulting  work  includes  the  copyright  notice
"Portions Copyright (c) 1989, 1990 James  A.   Roskind".  Derived
products  must  also  provide  the notice "Portions Copyright (c)
1989, 1990 James A.  Roskind" in  a  manner  appropriate  to  the
utility,   and  in  keeping  with  copyright  law  (e.g.:  EITHER
displayed when first invoked/executed; OR displayed  continuously
on  display terminal; OR via placement in the object code in form
readable in a printout, with or near the title of the work, or at
the end of the file).  No royalties, licenses or  commissions  of
any  kind  are  required  to copy this file, its translations, or
derivative products, when the copies are made in compliance  with
this  notice.  Persons  or  corporations  that  do make copies in
compliance  with  this  notice  may  charge  whatever  price   is
agreeable  to  a buyer, for such copies or derivative works. THIS
FILE IS PROVIDED ``AS IS'' AND WITHOUT  ANY  EXPRESS  OR  IMPLIED
WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

James A. Roskind
Independent Consultant
516 Latania Palm Drive
Indialantic FL, 32903
(407)729-4348
jar@hq.ileaf.com
or ...!uunet!leafusa!jar

---end of copyright notice---


COMMENTS-

My  goal  is  to  see  software  developers adopt my C++ grammar as a
standard until such time as a better  standard  is  accessible.   The
only  way  to  get it to become a standard, is to be sure that people
know that derivations are based on a specific work.   The  intent  of
releasing  this Flex input file is to facilitate experimentation with
my C++ grammar. The intent  of  the  copyright  notice  is  to  allow
arbitrary  commercial and non-commercial use of this file, as long as
reference is given to my standardization effort.   Without  reference
to  a specific standard, many alternative grammars would develop.  By
referring to the standard, the C++ grammar is given publicity,  which
should  lead  to further use in compatible products and systems.  The
benefits  of  such  a  standard  to  commercial  products  (browsers,
beautifiers,  translators,  compilers,  ...) should be obvious to the
developers, in that other compatible products will  emerge,  and  the
value  of  all  conforming  products  will rise.  Most developers are
aware of the value of acquiring  a  fairly  complete  grammar  for  a
language,  and  the  copyright  notice (and the resulting affiliation
with my work) should not be too high a price to pay.  By copyrighting
my work, I have some minor control over what this standard is, and  I
can  (hopefully)  keep it from degrading without my approval.  I will
consistently attempt to provide upgraded grammars that are  compliant
with  the  current  art, and the ANSI C++ Committee recommendation in
particular.  A  developer  is  never  prevented  from  modifying  the
grammar or this file to improve it in whatever way is seen fit. There
is  also  no  restriction on the sale of copies, or derivative works,
providing the request in the copyright notice are satisfied.

If you are not "copying" my work, but  are  rather  only  abstracting
some of my work, an acknowledgment with references to such a standard
would  be  appreciated.  Specifically, agreements with my grammar and
its resolution of otherwise ambiguous constructs, should be noted.

Simply put: "make whatever use you would like of the grammar and this
file, but include the ``portions Copyright ...'' as  a  reference  to
this standard."


*/

/* Last modified 7/4/91, Version 2.0 */

/* File cpp5.l, becomes lex.yy.c after processing by FLEX  */

/* This file is a dramatically cut down version of the FLEX input file
used in  my  ANSI  C  Preprocessor.   The  executable  version  of  my
preprocessor  is  available on many platforms (shareware), but this is
the only source extract that is currently being distributed.   If  you
need   a   full   ANSI   C  preprocessor,  with  extensive  diagnostic
capabilities and customization facilities, please contact  me  at  the
addresses given above.  Current platforms include IBMPC (DOS/OS2), Sun
(SPARC   and  Motorola),  and  IBM  R/6000.   ...  end  of  commercial
announcement.
 
This file is being distributed to facilitate experimentation  and  use
of my C and C++ grammar.
 
 
Comment  removal  must  be done during the lexing, as context (such as
enclosure in string literals) must be  observed.   For  this  cut-down
lexer,  we  will  assume that comments have been removed (don't assume
this if you are writing a compiler or browser!).
 
 
For each IDENTIFIER like string that is found,  there  are  several
distinct interpretations that can be applied:
 
1)  The  preprocessor  may  interpret  the  string as a "keyword" in a
directive (eg: "pragma" or "include", "defined").
 
2) The parser may interpret the string as a keyword. (eg: "int").
 
3) Both parser and preprocessor may interpret the string as a  keyword
(eg: "if").
 
Since  this  file  is based on source that actually lexically analyses
text for both preprocessing and parsing, macro definitions  were  used
throughout.   The macro definitions supplied here have been customized
to a C++ parse only, and  all  preprocessor  keywords  are  passed  as
IDENTIFIER  or  TYPEDEFname.   Also, since there is no symbol table to
interrogate to decide whether a string  is  a  TYPEDEFname,  I  simply
assume  that  any  identifier beginning with an upper case letter is a
TYPEDEFname.  This hack  should  allow  you  to  check  out  how  code
segments  are  parsed  using my grammar.  Unfortunately, if you really
want to parse major league code, you have to write a symbol table, and
maintain appropriate scoping information.
 
 
 
*/


/* Included code before lex code */
/*************** Includes and Defines *****************************/


#include "y.tab.h" /* YACC generated definitions based on C++ grammar */

typedef char * YYSTYPE; /* interface with lexer: should be  in  header
                        file*/

extern char * yylval;  /*  We  will always point at the text of the lexeme.
          This makes it easy to print out nice trees when  YYDEBUG  is
          enabled.   (see  C++  grammar  file  and  its  definition of
          YYDEBUG_LEXER_TEXT to be "yylval" */

#include <stdlib.h>
#include <string.h>

/* Prototypes */


#define WHITE_RETURN(x) /* do nothing */

#define NEW_LINE_RETURN() WHITE_RETURN('\n')

#define PA_KEYWORD_RETURN(x)   RETURN_VAL(x)  /* standard C PArser Keyword */
#define CPP_KEYWORD_RETURN(x)  PA_KEYWORD_RETURN(x)  /* C++ keyword */
#define PPPA_KEYWORD_RETURN(x) RETURN_VAL(x)  /* both PreProcessor and PArser keyword */
#define PP_KEYWORD_RETURN(x)   IDENTIFIER_RETURN()

#define IDENTIFIER_RETURN() RETURN_VAL(IDENTIFIER)

#define PPOP_RETURN(x)       RETURN_VAL((int)*yytext) /* PreProcess and Parser operator */
#define NAMED_PPOP_RETURN(x) RETURN_VAL(x)
#define ASCIIOP_RETURN(x)    RETURN_VAL((int)*yytext) /* a single character operator */
#define NAMEDOP_RETURN(x)    RETURN_VAL(x)            /* a multichar operator, with a name */

#define NUMERICAL_RETURN(x) RETURN_VAL(x)            /* some sort of constant */
#define LITERAL_RETURN(x)   RETURN_VAL(x)            /* a string literal */
#define C_COMMENT_RETURN(x) RETURN_VAL(x)      /* C Style comment  */
#define RETURN_VAL(x) return(x);
#define yywrap() 1




/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int yywrap YY_PROTO(( void ));
#else
extern int yywrap YY_PROTO(( void ));
#endif
#endif


#ifndef yytext_ptr
static void yy_flex_strncpy YY_PROTO(( char *, yyconst char *, int ));
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen YY_PROTO(( yyconst char * ));
#endif

#ifndef YY_NO_INPUT
#endif

#if YY_STACK_USED
static int yy_start_stack_ptr = 0;
static int yy_start_stack_depth = 0;
static int *yy_start_stack = 0;
#ifndef YY_NO_PUSH_STATE
static void yy_push_state YY_PROTO(( int new_state ));
#endif
#ifndef YY_NO_POP_STATE
static void yy_pop_state YY_PROTO(( void ));
#endif
#ifndef YY_NO_TOP_STATE
static int yy_top_state YY_PROTO(( void ));
#endif

#else
#define YY_NO_PUSH_STATE 1
#define YY_NO_POP_STATE 1
#define YY_NO_TOP_STATE 1
#endif

#ifdef YY_MALLOC_DECL
YY_MALLOC_DECL
#else
#if __STDC__
#ifndef __cplusplus
#include <stdlib.h>
#endif
#else 
/* Just try to get by without declaring the routines.  This will fail
* miserably on non-ANSI systems for which sizeof(size_t) != sizeof(int)
* or sizeof(void*) != sizeof(int).
*/
#endif
#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */

#ifndef ECHO
#define ECHO LexerOutput( yytext, yyleng )
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( (result = LexerInput( (char *) buf, max_size )) < 0 ) \
		YY_FATAL_ERROR( "input in flex scanner failed" );
#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) LexerError( msg )
#endif

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL int yyFlexLexer::yylex()
#endif

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	YY_USER_ACTION

YY_DECL
{
    register yy_state_type yy_current_state;
    register char *yy_cp, *yy_bp;
    register int yy_act;




    if ( yy_init )
    {
    yy_init = 0;

#ifdef YY_USER_INIT

    YY_USER_INIT;
#endif

    if ( ! yy_start )
		    yy_start = 1; /* first start state */

	    if ( ! yyin )
		    yyin = &std::cin;

	    if ( ! yyout )
		    yyout = &std::cout;

	    if ( ! yy_current_buffer )
		    yy_current_buffer =
		        yy_create_buffer( yyin, YY_BUF_SIZE );

	    yy_load_buffer_state();
    }

    while ( 1 )  /* loops until end-of-file is reached */
    {
    yy_cp = yy_c_buf_p;

    /* Support of yytext. */
    *yy_cp = yy_hold_char;

    /* yy_bp points to the position in yy_ch_buf of the start of
     * the current run.
     */
    yy_bp = yy_cp;

    yy_current_state = yy_start;
    yy_state_ptr = yy_state_buf;
    *yy_state_ptr++ = yy_current_state;
    yy_match:
    do
    {
	    register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
		    while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		    {
			    yy_current_state = (int) yy_def[yy_current_state];
			    if ( yy_current_state >= 364 )
				    yy_c = yy_meta[(unsigned int) yy_c];
		    }
		    yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		    *yy_state_ptr++ = yy_current_state;
		    ++yy_cp;
	    }
	    while ( yy_base[yy_current_state] != 571 );

    yy_find_action:
	    yy_current_state = *--yy_state_ptr;
	    yy_lp = yy_accept[yy_current_state];
    find_rule:  /* we branch to this label when backing up */
	    for ( ; ; ) /* until we find what rule we matched */
	    {
		    if ( yy_lp && yy_lp < yy_accept[yy_current_state + 1] )
		    {
			    yy_act = yy_acclist[yy_lp];
			    {
				    yy_full_match = yy_cp;
				    break;
			    }
		    }
		    --yy_cp;
		    yy_current_state = *--yy_state_ptr;
		    yy_lp = yy_accept[yy_current_state];
	    }

	    YY_DO_BEFORE_ACTION;

	    if ( yy_act != YY_END_OF_BUFFER )
	    {
		    int yyl;
		    for ( yyl = 0; yyl < yyleng; ++yyl )
			    if ( yytext[yyl] == '\n' )
				    ++yylineno;
	    }

    do_action:  /* This label is used only to access EOF actions. */


	    switch ( yy_act )
	    { /* beginning of action switch */
	    case 1:
		    YY_RULE_SETUP
		    {
		        /* Handle C-Style comment */
		        register int c;
		        if (m_keepComments)
		        m_comment += _U(yytext);
		        for ( ; ; )
		        {
			        while ( (c = yyinput()) != '*' && c != EOF )
				        {
					        if (m_keepComments)
						        m_comment += c;

					        ; /* eat up text of comment */
				        }
				        if (m_keepComments)
					        m_comment += c;
				        if ( c == '*' )
				        {
					        while ( (c = yyinput()) == '*' )
					        {
						        if (m_keepComments)
							        m_comment += c;
						        ;
					        }
					        if (m_keepComments)
						        m_comment += c;
					        if ( c == '/' )
					        {
						        break; /* found the end */
					        }
				        }
				        if ( c == EOF )
				        {
					        break;
				        }
			        }
		        if (m_keepComments )
		        return CComment;
		    }
		    YY_BREAK
	    case 2:
	    YY_RULE_SETUP
	    {
	        /* Handle CPP style comment    */
	        /* Constume everything until the newline */
	        register int c;
	        if (m_keepComments)
		        m_comment += _U(yytext);
		        while ((c = yyinput()) != '\n' && c != EOF)
		        {
			        if (m_keepComments)
					        m_comment += c;
			        }
		        if (m_keepComments)
		        m_comment += c;
		        if (m_keepComments )
			        return CPPComment;
			    }
			    YY_BREAK
		    case 3:
	    YY_RULE_SETUP
	    {
	        if (!m_returnWhite)
		        WHITE_RETURN(' ');
			        else
				        return ' ';

				    }
				    YY_BREAK
			    case 4:
	    YY_RULE_SETUP
	    {
	        if (!m_returnWhite)
		        WHITE_RETURN(' ');
			        else
				        return ' ';
				    }
				    YY_BREAK
			    case 5:
	    YY_RULE_SETUP
	    {
	        if (!m_returnWhite)
		        NEW_LINE_RETURN();
			        else
				        return '\n';
				    }
				    YY_BREAK
			    case 6:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(AUTO);
	    }
	    YY_BREAK
    case 7:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(BREAK);
	    }
	    YY_BREAK
    case 8:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(CASE);
	    }
	    YY_BREAK
    case 9:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(lexCHAR);
	    }
	    YY_BREAK
    case 10:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(lexCONST);
	    }
	    YY_BREAK
    case 11:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(CONTINUE);
	    }
	    YY_BREAK
    case 12:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(lexDEFAULT);
	    }
	    YY_BREAK
    case 13:
	    YY_RULE_SETUP
	    {PP_KEYWORD_RETURN(DEFINE);
	    }
	    YY_BREAK
    case 14:
	    YY_RULE_SETUP
	    {PP_KEYWORD_RETURN(OPDEFINED);
	    }
	    YY_BREAK
    case 15:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(DO);
	    }
	    YY_BREAK
    case 16:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(lexDOUBLE);
	    }
	    YY_BREAK
    case 17:
	    YY_RULE_SETUP
	    {PP_KEYWORD_RETURN(ELIF);
	    }
	    YY_BREAK
    case 18:
	    YY_RULE_SETUP
	    {PPPA_KEYWORD_RETURN(ELSE);
	    }
	    YY_BREAK
    case 19:
	    YY_RULE_SETUP
	    {PP_KEYWORD_RETURN(ENDIF);
	    }
	    YY_BREAK
    case 20:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(lexENUM);
	    }
	    YY_BREAK
    case 21:
	    YY_RULE_SETUP
	    {PP_KEYWORD_RETURN(ERROR);
	    }
	    YY_BREAK
    case 22:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(EXTERN);
	    }
	    YY_BREAK
    case 23:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(lexFLOAT);
	    }
	    YY_BREAK
    case 24:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(FOR);
	    }
	    YY_BREAK
    case 25:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(GOTO);
	    }
	    YY_BREAK
    case 26:
	    YY_RULE_SETUP
	    {PPPA_KEYWORD_RETURN(IF);
	    }
	    YY_BREAK
    case 27:
	    YY_RULE_SETUP
	    {PP_KEYWORD_RETURN(IFDEF);
	    }
	    YY_BREAK
    case 28:
	    YY_RULE_SETUP
	    {PP_KEYWORD_RETURN(IFNDEF);
	    }
	    YY_BREAK
    case 29:
	    YY_RULE_SETUP
	    {PP_KEYWORD_RETURN(INCLUDE);
	    }
	    YY_BREAK
    case 30:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(lexINT);
	    }
	    YY_BREAK
    case 31:
	    YY_RULE_SETUP
	    {PP_KEYWORD_RETURN(LINE);
	    }
	    YY_BREAK
    case 32:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(lexLONG);
	    }
	    YY_BREAK
    case 33:
	    YY_RULE_SETUP
	    {PP_KEYWORD_RETURN(PRAGMA);
	    }
	    YY_BREAK
    case 34:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(REGISTER);
	    }
	    YY_BREAK
    case 35:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(RETURN);
	    }
	    YY_BREAK
    case 36:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(lexSHORT);
	    }
	    YY_BREAK
    case 37:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(SIGNED);
	    }
	    YY_BREAK
    case 38:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(SIZEOF);
	    }
	    YY_BREAK
    case 39:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(STATIC);
	    }
	    YY_BREAK
    case 40:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(lexSTRUCT);
	    }
	    YY_BREAK
    case 41:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(SWITCH);
	    }
	    YY_BREAK
    case 42:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(TYPEDEF);
	    }
	    YY_BREAK
    case 43:
	    YY_RULE_SETUP
	    {PP_KEYWORD_RETURN(UNDEF);
	    }
	    YY_BREAK
    case 44:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(UNION);
	    }
	    YY_BREAK
    case 45:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(UNSIGNED);
	    }
	    YY_BREAK
    case 46:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(lexVOID);
	    }
	    YY_BREAK
    case 47:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(VOLATILE);
	    }
	    YY_BREAK
    case 48:
	    YY_RULE_SETUP
	    {PA_KEYWORD_RETURN(WHILE);
	    }
	    YY_BREAK
    case 49:
	    YY_RULE_SETUP
	    {CPP_KEYWORD_RETURN(lexCLASS);
	    }
	    YY_BREAK
    case 50:
	    YY_RULE_SETUP
	    {CPP_KEYWORD_RETURN(lexNAMESPACE);
	    }
	    YY_BREAK
    case 51:
	    YY_RULE_SETUP
	    {CPP_KEYWORD_RETURN(lexDELETE);
	    }
	    YY_BREAK
    case 52:
	    YY_RULE_SETUP
	    {CPP_KEYWORD_RETURN(FRIEND);
	    }
	    YY_BREAK
    case 53:
	    YY_RULE_SETUP
	    {CPP_KEYWORD_RETURN(INLINE);
	    }
	    YY_BREAK
    case 54:
	    YY_RULE_SETUP
	    {CPP_KEYWORD_RETURN(NEW);
	    }
	    YY_BREAK
    case 55:
	    YY_RULE_SETUP
	    {CPP_KEYWORD_RETURN(lexOPERATOR);
	    }
	    YY_BREAK
    case 56:
	    YY_RULE_SETUP
	    {CPP_KEYWORD_RETURN(OVERLOAD);
	    }
	    YY_BREAK
    case 57:
	    YY_RULE_SETUP
	    {CPP_KEYWORD_RETURN(lexPROTECTED);
	    }
	    YY_BREAK
    case 58:
	    YY_RULE_SETUP
	    {CPP_KEYWORD_RETURN(lexPRIVATE);
	    }
	    YY_BREAK
    case 59:
	    YY_RULE_SETUP
	    {CPP_KEYWORD_RETURN(lexPUBLIC);
	    }
	    YY_BREAK
    case 60:
	    YY_RULE_SETUP
	    {CPP_KEYWORD_RETURN(lexTHIS);
	    }
	    YY_BREAK
    case 61:
	    YY_RULE_SETUP
	    {CPP_KEYWORD_RETURN(VIRTUAL);
	    }
	    YY_BREAK
    case 62:
	    YY_RULE_SETUP
	    {IDENTIFIER_RETURN();
	    }
	    YY_BREAK
    case 63:
	    YY_RULE_SETUP
	    {NUMERICAL_RETURN(INTEGERconstant);
	    }
	    YY_BREAK
    case 64:
	    YY_RULE_SETUP
	    {NUMERICAL_RETURN(OCTALconstant);
	    }
	    YY_BREAK
    case 65:
	    YY_RULE_SETUP
	    {NUMERICAL_RETURN(HEXconstant);
	    }
	    YY_BREAK
    case 66:
	    YY_RULE_SETUP
	    {NUMERICAL_RETURN(FLOATINGconstant);
	    }
	    YY_BREAK
    case 67:
	    YY_RULE_SETUP
	    {
	        NUMERICAL_RETURN(CHARACTERconstant);
	    }
	    YY_BREAK
    case 68:
	    YY_RULE_SETUP
	    {
	        LITERAL_RETURN(STRINGliteral);
	    }
	    YY_BREAK
    case 69:
	    YY_RULE_SETUP
	    {PPOP_RETURN(LP);
	    }
	    YY_BREAK
    case 70:
	    YY_RULE_SETUP
	    {PPOP_RETURN(RP);
	    }
	    YY_BREAK
    case 71:
	    YY_RULE_SETUP
	    {PPOP_RETURN(COMMA);
	    }
	    YY_BREAK
    case 72:
	    YY_RULE_SETUP
	    {NAMED_PPOP_RETURN('#') ;
	    }
	    YY_BREAK
    case 73:
	    YY_RULE_SETUP
	    {NAMED_PPOP_RETURN(POUNDPOUND);
	    }
	    YY_BREAK
    case 74:
	    YY_RULE_SETUP
	    {ASCIIOP_RETURN(LC);
	    }
	    YY_BREAK
    case 75:
	    YY_RULE_SETUP
	    {ASCIIOP_RETURN(RC);
	    }
	    YY_BREAK
    case 76:
	    YY_RULE_SETUP
	    {ASCIIOP_RETURN(LB);
	    }
	    YY_BREAK
    case 77:
	    YY_RULE_SETUP
	    {ASCIIOP_RETURN(RB);
	    }
	    YY_BREAK
    case 78:
	    YY_RULE_SETUP
	    {ASCIIOP_RETURN(DOT);
	    }
	    YY_BREAK
    case 79:
	    YY_RULE_SETUP
	    {ASCIIOP_RETURN(AND);
	    }
	    YY_BREAK
    case 80:
	    YY_RULE_SETUP
	    {ASCIIOP_RETURN(STAR);
	    }
	    YY_BREAK
    case 81:
	    YY_RULE_SETUP
	    {ASCIIOP_RETURN(PLUS);
	    }
	    YY_BREAK
    case 82:
	    YY_RULE_SETUP
	    {ASCIIOP_RETURN(MINUS);
	    }
	    YY_BREAK
    case 83:
	    YY_RULE_SETUP
	    {ASCIIOP_RETURN(NEGATE);
	    }
	    YY_BREAK
    case 84:
	    YY_RULE_SETUP
	    {ASCIIOP_RETURN(NOT);
	    }
	    YY_BREAK
    case 85:
	    YY_RULE_SETUP
	    {ASCIIOP_RETURN(DIV);
	    }
	    YY_BREAK
    case 86:
	    YY_RULE_SETUP
	    {ASCIIOP_RETURN(MOD);
	    }
	    YY_BREAK
    case 87:
	    YY_RULE_SETUP
	    {ASCIIOP_RETURN(LT);
	    }
	    YY_BREAK
    case 88:
	    YY_RULE_SETUP
	    {ASCIIOP_RETURN(GT);
	    }
	    YY_BREAK
    case 89:
	    YY_RULE_SETUP
	    {ASCIIOP_RETURN(XOR);
	    }
	    YY_BREAK
    case 90:
	    YY_RULE_SETUP
	    {ASCIIOP_RETURN(PIPE);
	    }
	    YY_BREAK
    case 91:
	    YY_RULE_SETUP
	    {ASCIIOP_RETURN(QUESTION);
	    }
	    YY_BREAK
    case 92:
	    YY_RULE_SETUP
	    {ASCIIOP_RETURN(COLON);
	    }
	    YY_BREAK
    case 93:
	    YY_RULE_SETUP
	    {ASCIIOP_RETURN(SEMICOLON);
	    }
	    YY_BREAK
    case 94:
	    YY_RULE_SETUP
	    {ASCIIOP_RETURN(ASSIGN);
	    }
	    YY_BREAK
    case 95:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(DOTstar);
	    }
	    YY_BREAK
    case 96:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(CLCL);
	    }
	    YY_BREAK
    case 97:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(lexARROW);
	    }
	    YY_BREAK
    case 98:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(ARROWstar);
	    }
	    YY_BREAK
    case 99:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(ICR);
	    }
	    YY_BREAK
    case 100:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(DECR);
	    }
	    YY_BREAK
    case 101:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(LS);
	    }
	    YY_BREAK
    case 102:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(RS);
	    }
	    YY_BREAK
    case 103:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(LE);
	    }
	    YY_BREAK
    case 104:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(GE);
	    }
	    YY_BREAK
    case 105:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(EQ);
	    }
	    YY_BREAK
    case 106:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(NE);
	    }
	    YY_BREAK
    case 107:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(ANDAND);
	    }
	    YY_BREAK
    case 108:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(OROR);
	    }
	    YY_BREAK
    case 109:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(MULTassign);
	    }
	    YY_BREAK
    case 110:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(DIVassign);
	    }
	    YY_BREAK
    case 111:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(MODassign);
	    }
	    YY_BREAK
    case 112:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(PLUSassign);
	    }
	    YY_BREAK
    case 113:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(MINUSassign);
	    }
	    YY_BREAK
    case 114:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(LSassign);
	    }
	    YY_BREAK
    case 115:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(RSassign);
	    }
	    YY_BREAK
    case 116:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(ANDassign);
	    }
	    YY_BREAK
    case 117:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(ERassign);
	    }
	    YY_BREAK
    case 118:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(ORassign);
	    }
	    YY_BREAK
    case 119:
	    YY_RULE_SETUP
	    {NAMEDOP_RETURN(ELLIPSIS);
	    }
	    YY_BREAK
    case 120:
	    YY_RULE_SETUP
	    { return yytext[0];
	    }
	    YY_BREAK
    case 121:
	    YY_RULE_SETUP
	    ECHO;
	    YY_BREAK
    case YY_STATE_EOF(INITIAL):
					    yyterminate();

	    case YY_END_OF_BUFFER:
    {
			    /* Amount of text matched not including the EOB char. */
			    int yy_amount_of_matched_text = (int) (yy_cp - yytext_ptr) - 1;

			    /* Undo the effects of YY_DO_BEFORE_ACTION. */
			    *yy_cp = yy_hold_char;
			    YY_RESTORE_YY_MORE_OFFSET

			    if ( yy_current_buffer->yy_buffer_status == YY_BUFFER_NEW )
			    {
				    /* We're scanning a new file or input source.  It's
				     * possible that this happened because the user
				     * just pointed yyin at a new source and called
				     * yylex().  If so, then we have to assure
				     * consistency between yy_current_buffer and our
				     * globals.  Here is the right place to do so, because
				     * this is the first action (other than possibly a
				     * back-up) that will match for the new input source.
				     */
				    yy_n_chars = yy_current_buffer->yy_n_chars;
				    yy_current_buffer->yy_input_file = yyin;
				    yy_current_buffer->yy_buffer_status = YY_BUFFER_NORMAL;
			    }

			    /* Note that here we test for yy_c_buf_p "<=" to the position
			     * of the first EOB in the buffer, since yy_c_buf_p will
			     * already have been incremented past the NUL character
			     * (since all states make transitions on EOB to the
			     * end-of-buffer state).  Contrast this with the test
			     * in input().
			     */
			    if ( yy_c_buf_p <= &yy_current_buffer->yy_ch_buf[yy_n_chars] )
			    { /* This was really a NUL. */
				    yy_state_type yy_next_state;

				    yy_c_buf_p = yytext_ptr + yy_amount_of_matched_text;

				    yy_current_state = yy_get_previous_state();

				    /* Okay, we're now positioned to make the NUL
				     * transition.  We couldn't have
				     * yy_get_previous_state() go ahead and do it
				     * for us because it doesn't know how to deal
				     * with the possibility of jamming (and we don't
				     * want to build jamming into it because then it
				     * will run more slowly).
				     */

				    yy_next_state = yy_try_NUL_trans( yy_current_state );

				    yy_bp = yytext_ptr + YY_MORE_ADJ;

				    if ( yy_next_state )
				    {
					    /* Consume the NUL. */
					    yy_cp = ++yy_c_buf_p;
					    yy_current_state = yy_next_state;
					    goto yy_match;
				    }

				    else
				    {
					    yy_cp = yy_c_buf_p;
					    goto yy_find_action;
				    }
			    }

			    else
				    switch ( yy_get_next_buffer() )
				    {
				    case EOB_ACT_END_OF_FILE:
					    {
						    yy_did_buffer_switch_on_eof = 0;

						    if ( yywrap() )
						    {
							    /* Note: because we've taken care in
							     * yy_get_next_buffer() to have set up
							     * yytext, we can now set up
							     * yy_c_buf_p so that if some total
							     * hoser (like flex itself) wants to
							     * call the scanner after we return the
							     * YY_NULL, it'll still work - another
							     * YY_NULL will get returned.
							     */
							    yy_c_buf_p = yytext_ptr + YY_MORE_ADJ;

							    yy_act = YY_STATE_EOF(YY_START);
							    goto do_action;
						    }

						    else
						    {
							    if ( ! yy_did_buffer_switch_on_eof )
								    YY_NEW_FILE;
						    }
						    break;
					    }

				    case EOB_ACT_CONTINUE_SCAN:
					    yy_c_buf_p =
					        yytext_ptr + yy_amount_of_matched_text;

					    yy_current_state = yy_get_previous_state();

					    yy_cp = yy_c_buf_p;
					    yy_bp = yytext_ptr + YY_MORE_ADJ;
					    goto yy_match;

				    case EOB_ACT_LAST_MATCH:
					    yy_c_buf_p =
					        &yy_current_buffer->yy_ch_buf[yy_n_chars];

					    yy_current_state = yy_get_previous_state();

					    yy_cp = yy_c_buf_p;
					    yy_bp = yytext_ptr + YY_MORE_ADJ;
					    goto yy_find_action;
				    }
			    break;
		    }

	    default:
		    YY_FATAL_ERROR(
		        "fatal flex scanner internal error--no action found" );
	    } /* end of action switch */
    } /* end of scanning one token */
} /* end of yylex */

yyFlexLexer::yyFlexLexer( std::istream* arg_yyin, std::ostream* arg_yyout )
{
	yyin = arg_yyin;
	yyout = arg_yyout;
	yy_c_buf_p = 0;
	yy_init = 1;
	yy_start = 0;
	yy_flex_debug = 0;
	yylineno = 1; // this will only get updated if %option yylineno

	yy_did_buffer_switch_on_eof = 0;

	yy_looking_for_trail_begin = 0;
	yy_more_flag = 0;
	yy_more_len = 0;
	yy_more_offset = yy_prev_more_offset = 0;

	yy_start_stack_ptr = yy_start_stack_depth = 0;
	yy_start_stack = 0;

	yy_current_buffer = 0;

#ifdef YY_USES_REJECT

	yy_state_buf = new yy_state_type[YY_BUF_SIZE + 2];
#else

	yy_state_buf = 0;
#endif

}

yyFlexLexer::~yyFlexLexer()
{
	delete[] yy_state_buf;
	yy_delete_buffer( yy_current_buffer );
}

void yyFlexLexer::switch_streams( std::istream* new_in, std::ostream* new_out )
{
	if ( new_in )
	{
		yy_delete_buffer( yy_current_buffer );
		yy_switch_to_buffer( yy_create_buffer( new_in, YY_BUF_SIZE ) );
	}

	if ( new_out )
		yyout = new_out;
}

#ifdef YY_INTERACTIVE
int yyFlexLexer::LexerInput( char* buf, int /* max_size */ )
#else
int yyFlexLexer::LexerInput( char* buf, int max_size )
#endif
{
	if ( yyin->eof() || yyin->fail() )
		return 0;

#ifdef YY_INTERACTIVE

	yyin->get
	( buf[0] );

	if ( yyin->eof() )
		return 0;

	if ( yyin->bad() )
		return -1;

	return 1;

#else

	(void) yyin->read( buf, max_size );

	if ( yyin->bad() )
		return -1;
	else
		return yyin->gcount();
#endif

}

void yyFlexLexer::LexerOutput( const char* buf, int size )
{
	(void) yyout->write( buf, size );
}

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 * EOB_ACT_LAST_MATCH -
 * EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 * EOB_ACT_END_OF_FILE - end of file
 */

int yyFlexLexer::yy_get_next_buffer()
{
	register char *dest = yy_current_buffer->yy_ch_buf;
	register char *source = yytext_ptr;
	register int number_to_move, i;
	int ret_val;

	if ( yy_c_buf_p > &yy_current_buffer->yy_ch_buf[yy_n_chars + 1] )
		YY_FATAL_ERROR(
		    "fatal flex scanner internal error--end of buffer missed" );

	if ( yy_current_buffer->yy_fill_buffer == 0 )
	{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( yy_c_buf_p - yytext_ptr - YY_MORE_ADJ == 1 )
		{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */ 
			return EOB_ACT_END_OF_FILE;
		}

		else
		{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */ 
			return EOB_ACT_LAST_MATCH;
		}
	}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) (yy_c_buf_p - yytext_ptr) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( yy_current_buffer->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		yy_current_buffer->yy_n_chars = yy_n_chars = 0;

	else
	{
		int num_to_read =
		    yy_current_buffer->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
		{ /* Not enough room in the buffer - grow it. */
#if	0
			YY_FATAL_ERROR(
			    "input buffer overflow, can't enlarge buffer because scanner uses REJECT" );
#else

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = yy_current_buffer;

			int yy_c_buf_p_offset =
				(int) (yy_c_buf_p - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
			{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
							   /* Include room in for 2 EOB chars. */
							   yy_flex_realloc( (void *) b->yy_ch_buf,
												b->yy_buf_size + 2 );
			}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = 0;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
					"fatal error - scanner input buffer overflow" );

			yy_c_buf_p = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = yy_current_buffer->yy_buf_size -
						  number_to_move - 1;
#endif

		}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&yy_current_buffer->yy_ch_buf[number_to_move]),
		          yy_n_chars, num_to_read );

		yy_current_buffer->yy_n_chars = yy_n_chars;
	}

	if ( yy_n_chars == 0 )
	{
		if ( number_to_move == YY_MORE_ADJ )
		{
			ret_val = EOB_ACT_END_OF_FILE;
			yyrestart( yyin );
		}

		else
		{
			ret_val = EOB_ACT_LAST_MATCH;
			yy_current_buffer->yy_buffer_status =
			    YY_BUFFER_EOF_PENDING;
		}
	}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	yy_n_chars += number_to_move;
	yy_current_buffer->yy_ch_buf[yy_n_chars] = YY_END_OF_BUFFER_CHAR;
	yy_current_buffer->yy_ch_buf[yy_n_chars + 1] = YY_END_OF_BUFFER_CHAR;

	yytext_ptr = &yy_current_buffer->yy_ch_buf[0];

	return ret_val;
}


/* yy_get_previous_state - get the state just before the EOB char was reached */

yy_state_type yyFlexLexer::yy_get_previous_state()
{
	register yy_state_type yy_current_state;
	register char *yy_cp;

	yy_current_state = yy_start;
	yy_state_ptr = yy_state_buf;
	*yy_state_ptr++ = yy_current_state;

	for ( yy_cp = yytext_ptr + YY_MORE_ADJ; yy_cp < yy_c_buf_p; ++yy_cp )
	{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 364 )
				yy_c = yy_meta[(unsigned int) yy_c];
		}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		*yy_state_ptr++ = yy_current_state;
	}

	return yy_current_state;
}


/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 * next_state = yy_try_NUL_trans( current_state );
 */

yy_state_type yyFlexLexer::yy_try_NUL_trans( yy_state_type yy_current_state )
{
	register int yy_is_jam;

	register YY_CHAR yy_c = 1;
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
	{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 364 )
			yy_c = yy_meta[(unsigned int) yy_c];
	}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 363);
	if ( ! yy_is_jam )
		*yy_state_ptr++ = yy_current_state;

	return yy_is_jam ? 0 : yy_current_state;
}


void yyFlexLexer::yyunput( int c, register char* yy_bp )
{
	register char *yy_cp = yy_c_buf_p;

	/* undo effects of setting up yytext */
	*yy_cp = yy_hold_char;

	if ( yy_cp < yy_current_buffer->yy_ch_buf + 2 )
	{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		register int number_to_move = yy_n_chars + 2;
		register char *dest = &yy_current_buffer->yy_ch_buf[
		                          yy_current_buffer->yy_buf_size + 2];
		register char *source =
		    &yy_current_buffer->yy_ch_buf[number_to_move];

		while ( source > yy_current_buffer->yy_ch_buf )
			*--dest = *--source;

		yy_cp += (int) (dest - source);
		yy_bp += (int) (dest - source);
		yy_current_buffer->yy_n_chars =
		    yy_n_chars = yy_current_buffer->yy_buf_size;

		if ( yy_cp < yy_current_buffer->yy_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
	}

	*--yy_cp = (char) c;

	if ( c == '\n' )
		--yylineno;

	yytext_ptr = yy_bp;
	yy_hold_char = *yy_cp;
	yy_c_buf_p = yy_cp;
}


int yyFlexLexer::yyinput()
{
	int c;

	*yy_c_buf_p = yy_hold_char;

	if ( *yy_c_buf_p == YY_END_OF_BUFFER_CHAR )
	{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( yy_c_buf_p < &yy_current_buffer->yy_ch_buf[yy_n_chars] )
			/* This was really a NUL. */
			*yy_c_buf_p = '\0';

		else
		{ /* need more input */
			int offset = yy_c_buf_p - yytext_ptr;
			++yy_c_buf_p;

			switch ( yy_get_next_buffer() )
			{
			case EOB_ACT_LAST_MATCH:
				/* This happens because yy_g_n_b()
				 * sees that we've accumulated a
				 * token and flags that we need to
				 * try matching the token before
				 * proceeding.  But for input(),
				 * there's no matching to consider.
				 * So convert the EOB_ACT_LAST_MATCH
				 * to EOB_ACT_END_OF_FILE.
				 */

				/* Reset buffer status. */
				yyrestart( yyin );

				/* fall through */

			case EOB_ACT_END_OF_FILE:
				{
					if ( yywrap() )
						return EOF;

					if ( ! yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
#ifdef __cplusplus

					return yyinput();
#else

					return input();
#endif

				}

			case EOB_ACT_CONTINUE_SCAN:
				yy_c_buf_p = yytext_ptr + offset;
				break;
			}
		}
	}

	c = *(unsigned char *) yy_c_buf_p; /* cast for 8-bit char's */
	*yy_c_buf_p = '\0'; /* preserve yytext */
	yy_hold_char = *++yy_c_buf_p;

	if ( c == '\n' )
		++yylineno;

	return c;
}


void yyFlexLexer::yyrestart( std::istream* input_file )
{
	if ( ! yy_current_buffer )
		yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE );

	yy_init_buffer( yy_current_buffer, input_file );
	yy_load_buffer_state();
}


void yyFlexLexer::yy_switch_to_buffer( YY_BUFFER_STATE new_buffer )
{
	if ( yy_current_buffer == new_buffer )
		return ;

	if ( yy_current_buffer )
	{
		/* Flush out information for old buffer. */
		*yy_c_buf_p = yy_hold_char;
		yy_current_buffer->yy_buf_pos = yy_c_buf_p;
		yy_current_buffer->yy_n_chars = yy_n_chars;
	}

	yy_current_buffer = new_buffer;
	yy_load_buffer_state();

	/* We don't actually know whether we did this switch during
	 * EOF (yywrap()) processing, but the only time this flag
	 * is looked at is after yywrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	yy_did_buffer_switch_on_eof = 1;
}


void yyFlexLexer::yy_load_buffer_state()
{
	yy_n_chars = yy_current_buffer->yy_n_chars;
	yytext_ptr = yy_c_buf_p = yy_current_buffer->yy_buf_pos;
	yyin = yy_current_buffer->yy_input_file;
	yy_hold_char = *yy_c_buf_p;
}


YY_BUFFER_STATE yyFlexLexer::yy_create_buffer( std::istream* file, int size )
{
	YY_BUFFER_STATE b;

	b = (YY_BUFFER_STATE) yy_flex_alloc( sizeof( struct yy_buffer_state ) );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) yy_flex_alloc( b->yy_buf_size + 2 );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_is_our_buffer = 1;

	yy_init_buffer( b, file );

	return b;
}


void yyFlexLexer::yy_delete_buffer( YY_BUFFER_STATE b )
{
	if ( ! b )
		return ;

	if ( b == yy_current_buffer )
		yy_current_buffer = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		yy_flex_free( (void *) b->yy_ch_buf );

	yy_flex_free( (void *) b );
}


/*extern "C" int isatty YY_PROTO(( int ));*/
void yyFlexLexer::yy_init_buffer( YY_BUFFER_STATE b, std::istream* file )

{
	yy_flush_buffer( b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

	b->yy_is_interactive = 0;
}


void yyFlexLexer::yy_flush_buffer( YY_BUFFER_STATE b )
{
	if ( ! b )
		return ;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == yy_current_buffer )
		yy_load_buffer_state();
}


#ifndef YY_NO_SCAN_BUFFER
#endif


#ifndef YY_NO_SCAN_STRING
#endif


#ifndef YY_NO_SCAN_BYTES
#endif


#ifndef YY_NO_PUSH_STATE
void yyFlexLexer::yy_push_state( int new_state )
{
	if ( yy_start_stack_ptr >= yy_start_stack_depth )
	{
		yy_size_t new_size;

		yy_start_stack_depth += YY_START_STACK_INCR;
		new_size = yy_start_stack_depth * sizeof( int );

		if ( ! yy_start_stack )
			yy_start_stack = (int *) yy_flex_alloc( new_size );

		else
			yy_start_stack = (int *) yy_flex_realloc(
			                     (void *) yy_start_stack, new_size );

		if ( ! yy_start_stack )
			YY_FATAL_ERROR(
			    "out of memory expanding start-condition stack" );
	}

	yy_start_stack[yy_start_stack_ptr++] = YY_START;

	BEGIN(new_state);
}
#endif


#ifndef YY_NO_POP_STATE
void yyFlexLexer::yy_pop_state()
{
	if ( --yy_start_stack_ptr < 0 )
		YY_FATAL_ERROR( "start-condition stack underflow" );

	BEGIN(yy_start_stack[yy_start_stack_ptr]);
}
#endif


#ifndef YY_NO_TOP_STATE
int yyFlexLexer::yy_top_state()
{
	return yy_start_stack[yy_start_stack_ptr - 1];
}
#endif

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif


void yyFlexLexer::LexerError( yyconst char msg[] )
{
	std::cerr << msg << '\n';
	exit( YY_EXIT_FAILURE );
}


/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
	{ \
		/* Undo effects of setting up yytext. */ \
		yytext[yyleng] = yy_hold_char; \
		yy_c_buf_p = yytext + n; \
		yy_hold_char = *yy_c_buf_p; \
		*yy_c_buf_p = '\0'; \
		yyleng = n; \
	} \
	while ( 0 )


/* Internal utility routines. */

#ifndef yytext_ptr
#ifdef YY_USE_PROTOS
static void yy_flex_strncpy( char *s1, yyconst char *s2, int n )
#else
static void yy_flex_strncpy( s1, s2, n )
char *s1;
yyconst char *s2;
int n;
#endif

{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
#ifdef YY_USE_PROTOS
static int yy_flex_strlen( yyconst char *s )
#else
static int yy_flex_strlen( s )
yyconst char *s;
#endif

{
	register int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif


#ifdef YY_USE_PROTOS
static void *yy_flex_alloc( yy_size_t size )
#else
static void *yy_flex_alloc( size )
yy_size_t size;
#endif

{
	return (void *) malloc( size );
}

#ifdef YY_USE_PROTOS
static void *yy_flex_realloc( void *ptr, yy_size_t size )
#else
static void *yy_flex_realloc( ptr, size )
void *ptr;
yy_size_t size;
#endif

{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */ 
	return (void *) realloc( (char *) ptr, size );
}

#ifdef YY_USE_PROTOS
static void yy_flex_free( void *ptr )
#else
static void yy_flex_free( ptr )
void *ptr;
#endif

{
	free( ptr );
}

#if YY_MAIN
int main()
{
	yylex();
	return 0;
}
#endif



} // namespace flex


/*******************************************************************/
