//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// copyright            : (C) 2009 by Eran Ifrah
// file name            : subversion_strings.h
//
// -------------------------------------------------------------------------
// A
//              _____           _      _     _ _
//             /  __ \         | |    | |   (_) |
//             | /  \/ ___   __| | ___| |    _| |_ ___
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/
//              \____/\___/ \__,_|\___\_____/_|\__\___|
//
//                                                  F i l e
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#ifndef SVNSTRINGS_H
#define SVNSTRINGS_H

const wxString svnNO_FILES_TO_DISPLAY     = _("No Files to Display");
const wxString svnMODIFIED_FILES          = _("Modified Files");
const wxString svnADDED_FILES             = _("Added Files");
const wxString svnDELETED_FILES           = _("Deleted Files");
const wxString svnCONFLICTED_FILES        = _("Conflicted Files");
const wxString svnLOCKED_FILES            = _("Locked Files");
const wxString svnUNVERSIONED_FILES       = _("Unversioned Files");
const wxString svnCONSOLE_TEXT            = _("Svn");
const wxString svnANOTHER_PROCESS_RUNNING = _("\nMESSAGE: Another process is already running.\nMESSAGE: Ignoring last command.\n");
#endif // SVNSTRINGS_H
