//////////////////////////////////////////////////////////////////////
// This file was auto-generated by codelite's wxCrafter Plugin
// wxCrafter project file: newquickwatch.wxcp
// Do not modify this file by hand!
//////////////////////////////////////////////////////////////////////

#ifndef CL_LITEEDITOR_NEWQUICKWATCH_BASE_CLASSES_H
#define CL_LITEEDITOR_NEWQUICKWATCH_BASE_CLASSES_H

#include <wx/settings.h>
#include <wx/xrc/xmlres.h>
#include <wx/xrc/xh_bmp.h>
#include <wx/popupwin.h>
#include <wx/artprov.h>
#include <wx/sizer.h>
#include <wx/panel.h>
#include <wx/scrolwin.h>
#include <wx/treectrl.h>
#include <wx/statbmp.h>
#include <wx/dialog.h>
#include <wx/iconbndl.h>
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/button.h>
#if wxVERSION_NUMBER >= 2900
#include <wx/persist.h>
#include <wx/persist/toplevel.h>
#include <wx/persist/bookctrl.h>
#include <wx/persist/treebook.h>
#endif

class clDebuggerTipWindowBase : public wxPopupWindow
{
protected:
    wxPanel* m_panel46;
    wxScrolledWindow* m_scrollWin6;
    wxTreeCtrl* m_treeCtrl;
    wxPanel* m_panelStatusBar;
    wxStaticBitmap* m_staticBitmap44;

protected:
    virtual void OnKeyDown(wxKeyEvent& event) { event.Skip(); }
    virtual void OnLeftDown(wxMouseEvent& event) { event.Skip(); }
    virtual void OnMouseMove(wxMouseEvent& event) { event.Skip(); }
    virtual void OnItemExpanded(wxTreeEvent& event) { event.Skip(); }
    virtual void OnExpandItem(wxTreeEvent& event) { event.Skip(); }
    virtual void OnItemMenu(wxTreeEvent& event) { event.Skip(); }
    virtual void OnStatusLeftUp(wxMouseEvent& event) { event.Skip(); }
    virtual void OnStatuMotion(wxMouseEvent& event) { event.Skip(); }
    virtual void OnTipLeftDown(wxMouseEvent& event) { event.Skip(); }

public:
    wxTreeCtrl* GetTreeCtrl() { return m_treeCtrl; }
    wxScrolledWindow* GetScrollWin6() { return m_scrollWin6; }
    wxStaticBitmap* GetStaticBitmap44() { return m_staticBitmap44; }
    wxPanel* GetPanelStatusBar() { return m_panelStatusBar; }
    wxPanel* GetPanel46() { return m_panel46; }
    clDebuggerTipWindowBase(wxWindow* parent, long style = wxBORDER_NONE);
    virtual ~clDebuggerTipWindowBase();
};


class clDebuggerEditItemDlgBase : public wxDialog
{
protected:
    wxPanel* m_panel32;
    wxStaticText* m_staticText36;
    wxTextCtrl* m_textCtrl38;
    wxButton* m_button28;
    wxButton* m_button30;

protected:

public:
    wxStaticText* GetStaticText36() { return m_staticText36; }
    wxTextCtrl* GetTextCtrl38() { return m_textCtrl38; }
    wxPanel* GetPanel32() { return m_panel32; }
    wxButton* GetButton28() { return m_button28; }
    wxButton* GetButton30() { return m_button30; }
    clDebuggerEditItemDlgBase(wxWindow* parent, wxWindowID id = wxID_ANY, const wxString& title = _("Edit Item"), const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxSize(400,-1), long style = wxDEFAULT_DIALOG_STYLE|wxSTAY_ON_TOP);
    virtual ~clDebuggerEditItemDlgBase();
};

#endif
