//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Copyright            : (C) 2015 Eran Ifrah
// File name            : addincludefiledlgbase.h
//
// -------------------------------------------------------------------------
// A
//              _____           _      _     _ _
//             /  __ \         | |    | |   (_) |
//             | /  \/ ___   __| | ___| |    _| |_ ___
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/
//              \____/\___/ \__,_|\___\_____/_|\__\___|
//
//                                                  F i l e
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// This file was auto-generated by codelite's wxCrafter Plugin
// wxCrafter project file: AddIncludeFile.wxcp
// Do not modify this file by hand!
//////////////////////////////////////////////////////////////////////

#ifndef CODELITE_LITEEDITOR_ADDINCLUDEFILE_BASE_CLASSES_H
#define CODELITE_LITEEDITOR_ADDINCLUDEFILE_BASE_CLASSES_H

#include <wx/settings.h>
#include <wx/xrc/xmlres.h>
#include <wx/xrc/xh_bmp.h>
#include <wx/dialog.h>
#include <wx/iconbndl.h>
#include <wx/artprov.h>
#include <wx/sizer.h>
#include <wx/pen.h>
#include <wx/aui/auibar.h>
#include <map>
#include <wx/menu.h>
#include <wx/toolbar.h>
#include <wx/panel.h>
#include <wx/stc/stc.h>
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/button.h>
#if wxVERSION_NUMBER >= 2900
#include <wx/persist.h>
#include <wx/persist/toplevel.h>
#include <wx/persist/bookctrl.h>
#include <wx/persist/treebook.h>
#endif

class AddIncludeFileDlgBase : public wxDialog
{
public:
    enum {
        ID_LINEDOWN = 5001,
        ID_LINEUP = 5002,
    };
protected:
    wxAuiToolBar* m_auibar6;
    wxPanel* m_mainPanel;
    wxStyledTextCtrl* m_textCtrlPreview;
    wxStaticText* m_staticText21;
    wxStaticText* m_staticTextPreview;
    wxStaticText* m_staticText5;
    wxTextCtrl* m_textCtrlFullPath;
    wxStdDialogButtonSizer* m_stdBtnSizer23;
    wxButton* m_button25;
    wxButton* m_button27;

protected:
    virtual void OnButtonUp(wxCommandEvent& event) { event.Skip(); }
    virtual void OnButtonDown(wxCommandEvent& event) { event.Skip(); }
    virtual void OnClearCachedPaths(wxCommandEvent& event) { event.Skip(); }
    virtual void OnButtonOK(wxCommandEvent& event) { event.Skip(); }

public:
    wxAuiToolBar* GetAuibar6() { return m_auibar6; }
    wxStyledTextCtrl* GetTextCtrlPreview() { return m_textCtrlPreview; }
    wxStaticText* GetStaticText21() { return m_staticText21; }
    wxStaticText* GetStaticTextPreview() { return m_staticTextPreview; }
    wxStaticText* GetStaticText5() { return m_staticText5; }
    wxTextCtrl* GetTextCtrlFullPath() { return m_textCtrlFullPath; }
    wxPanel* GetMainPanel() { return m_mainPanel; }
    AddIncludeFileDlgBase(wxWindow* parent, wxWindowID id = wxID_ANY, const wxString& title = _("Add a missing header file"), const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxSize(-1,-1), long style = wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER);
    virtual ~AddIncludeFileDlgBase();
};

#endif
