//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Copyright            : (C) 2015 Eran Ifrah
// File name            : gdb_result_parser.h
//
// -------------------------------------------------------------------------
// A
//              _____           _      _     _ _
//             /  __ \         | |    | |   (_) |
//             | /  \/ ___   __| | ___| |    _| |_ ___
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/
//              \____/\___/ \__,_|\___\_____/_|\__\___|
//
//                                                  F i l e
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#define GDB_DONE 257
#define GDB_RUNNING 258
#define GDB_CONNECTED 259
#define GDB_ERROR 260
#define GDB_EXIT 261
#define GDB_STACK_ARGS 262
#define GDB_VALUE 263
#define GDB_ARGS 264
#define GDB_FRAME 265
#define GDB_NAME 266
#define GDB_STRING 267
#define GDB_LEVEL 268
#define GDB_ESCAPED_STRING 269
#define GDB_LOCALS 270
#define GDB_VARIABLES 271
#define GDB_INTEGER 272
#define GDB_OCTAL 273
#define GDB_HEX 274
#define GDB_FLOAT 275
#define GDB_IDENTIFIER 276
#define GDB_NUMCHILD 277
#define GDB_TYPE 278
#define GDB_DATA 279
#define GDB_ADDR 280
#define GDB_ASCII 281
#define GDB_CHILDREN 282
#define GDB_CHILD 283
#define GDB_MORE 284
#define GDB_VAROBJ 285
#define GDB_BREAKPOINT_TABLE 286
#define GDB_NR_ROWS 287
#define GDB_NR_COLS 288
#define GDB_HDR 289
#define GDB_BODY 290
#define GDB_BKPT 291
#define GDB_STOPPED 292
#define GDB_TIME 293
#define GDB_REASON 294
#define GDB_CHANGELIST 295
#define GDB_DISPLAYHINT 296
#define GDB_HAS_MORE 297
#define GDB_NEW_NUM_CHILDREN 298
#define GDB_NEW_CHILDREN 299
#define GDB_FUNC_NAME 300
#define GDB_OFFSET 301
#define GDB_INSTRUCTION 302
#define GDB_ADDRESS 303
#define GDB_ASM_INSTS 304
#define GDB_THREAD_GROUPS 305
#define GDB_REGISTER_NAMES 306
#define GDB_DYNAMIC 307
