#include "SFTPUploadDialog.h"

SFTPUploadDialog::SFTPUploadDialog(wxWindow* parent)
    : SFTPUploadDialogBase(parent)
{
}

SFTPUploadDialog::~SFTPUploadDialog()
{
}

void SFTPUploadDialog::OnOKUI(wxUpdateUIEvent& event)
{
}
