//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// copyright            : (C) 2008 by Eran Ifrah                            
// file name            : editcmpfileinfodlg.h              
//                                                                          
// -------------------------------------------------------------------------
// A                                                                        
//              _____           _      _     _ _                            
//             /  __ \         | |    | |   (_) |                           
//             | /  \/ ___   __| | ___| |    _| |_ ___                      
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )                     
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/                     
//              \____/\___/ \__,_|\___\_____/_|\__\___|                     
//                                                                          
//                                                  F i l e                 
//                                                                          
//    This program is free software; you can redistribute it and/or modify  
//    it under the terms of the GNU General Public License as published by  
//    the Free Software Foundation; either version 2 of the License, or     
//    (at your option) any later version.                                   
//                                                                          
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#ifndef __editcmpfileinfodlg__
#define __editcmpfileinfodlg__

/**
@file
Subclass of EditCmpFileInfoBase, which is generated by wxFormBuilder.
*/

#include "editcmpfileinfobase.h"

/** Implementing EditCmpFileInfoBase */
class EditCmpFileInfo : public EditCmpFileInfoBase
{
protected:
	// Handlers for EditCmpFileInfoBase events.
	void OnFileTypeText( wxCommandEvent& event );
	void OnCompilationLine( wxCommandEvent& event );
	void OnButtonOk( wxCommandEvent& event );
	void OnButtonOkUI( wxUpdateUIEvent& event );
	void OnButtonCancel( wxCommandEvent& event );
	void OnButtonHelp( wxCommandEvent& event );
public:
	/** Constructor */
	EditCmpFileInfo( wxWindow* parent);
	virtual ~EditCmpFileInfo();
	
	wxString GetCompilationLine() {
		return m_textCtrl2->GetValue();
	}
	wxString GetKind() {
		return m_choiceFileTypeIs->GetStringSelection();
	}
	wxString GetExtension() {
		return m_textCtrlFileType->GetValue();
	}

	void SetCompilationLine(const wxString &s) ;

	void SetExtension(const wxString &s) ;

	void SetKind(const wxString &s) ;
};

#endif // __editcmpfileinfodlg__
