<?php


Notice: Undefined index: internal in /home/eran/devl/codelite/codelitephp/php-plugin/doc-gen.php on line 85

/**
 * The full path and filename of the file with symlinks resolved. If used inside an include,
 *         the name of the included file is returned.
 * @link http://www.php.net/manual/en/language.constants.php
 */
define ('__FILE__', null);

/**
 * The current line number of the file.
 * @link http://www.php.net/manual/en/language.constants.php
 */
define ('__LINE__', null);

/**
 * The class name. The class name includes the namespace
 *         it was declared in (e.g. Foo\Bar).
 *         Note that as of PHP 5.4 __CLASS__ works also in traits. When used
 *         in a trait method, __CLASS__ is the name of the class the trait
 *         is used in.
 * @link http://www.php.net/manual/en/language.constants.php
 */
define ('__CLASS__', "");

/**
 * The function name.
 * @link http://www.php.net/manual/en/language.constants.php
 */
define ('__FUNCTION__', null);

/**
 * The class method name.
 * @link http://www.php.net/manual/en/language.constants.php
 */
define ('__METHOD__', null);

/**
 * The directory of the file.  If used inside an include,
 *         the directory of the included file is returned. This is equivalent
 *         to dirname(__FILE__). This directory name
 *         does not have a trailing slash unless it is the root directory.
 * @link http://www.php.net/manual/en/language.constants.php
 */
define ('__DIR__', null);

/**
 * The name of the current namespace.
 * @link http://www.php.net/manual/en/language.constants.php
 */
define ('__NAMESPACE__', null);
?>
