<?php

// Start of Zend Code Tracing v.8.0.0

function zend_codetracing_options () {}

function zend_codetracing_dump () {}

function zend_codetracing_event () {}

function zend_codetracing_dump_name () {}

function zend_codetracing_dump_base () {}

function zend_codetracing_dump_amf () {}

function zend_codetracing_dump_amf_file () {}

function zend_codetracing_dump_inspect () {}

define ('ZCT_TRACE_TIME', 1);
define ('ZCT_TRACE_SOURCE_LINES', 2);
define ('ZCT_TRACE_INTERNAL_FUNCTIONS', 4);
define ('ZCT_TRACE_USER_FUNCTIONS', 8);
define ('ZCT_TRACE_INCLUDES', 16);
define ('ZCT_TRACE_ARGUMENTS', 32);
define ('ZCT_TRACE_RETURN_VALUES', 64);
define ('ZCT_TRACE_EXCEPTIONS', 128);
define ('ZCT_TRACE_ARRAYS', 256);
define ('ZCT_TRACE_WRITE', 512);
define ('ZCT_TRACE_HEADERS', 1024);
define ('ZCT_TRACE_MEMORY_USAGE', 2048);
define ('ZCT_TRACE_ERRORS', 4096);
define ('ZCT_TRACE_DUMP_SEGV', 8192);
define ('ZCT_TRACE_EVENTS', 16384);
define ('ZCT_TRACE_ENABLED', 1073741824);
define ('ZCT_TRACE_DUMP', -2147483648);
define ('ZCT_DUMP_REASON_USER', 0);
define ('ZCT_DUMP_REASON_EVENT', 1);
define ('ZCT_DUMP_REASON_REQUEST', 2);
define ('ZCT_DUMP_REASON_SIGNAL', 3);

// End of Zend Code Tracing v.8.0.0
?>
