//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// copyright            : (C) 2008 by Eran Ifrah
// file name            : code_parser.rc
//
// -------------------------------------------------------------------------
// A
//              _____           _      _     _ _
//             /  __ \         | |    | |   (_) |
//             | /  \/ ___   __| | ___| |    _| |_ ___
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/
//              \____/\___/ \__,_|\___\_____/_|\__\___|
//
//                                                  F i l e
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

aaaaa ICON "res/codelite-logo.ico"
#include "wx/msw/wx.rc"
#if WIN64
1 24 "wx/msw/amd64.manifest"
#else
1 24 "wx/msw/wx.manifest"
#endif
///////////////////////////////////////////////////////////////////////////////
// Version information
VS_VERSION_INFO    VERSIONINFO
FILEVERSION      9,3,0,0
PRODUCTVERSION   9,3,0,0
FILEFLAGSMASK    0x3fL // VS_FFI_FILEFLAGSMASK
FILEFLAGS        0x0L  // final version
FILEOS           VOS_NT_WINDOWS32
FILETYPE         VFT_APP
FILESUBTYPE      VFT2_UNKNOWN { // not used
    BLOCK "StringFileInfo"
    {
        BLOCK "040904E4" // Lang=US English, CharSet=Windows Multilingual
        {
            VALUE "Comments",         "Licensed under GPL2."
            VALUE "CompanyName",      "Eran Ifrah"
            VALUE "Developer",        "Eran Ifrah"
            VALUE "FileDescription",  "CodeLite, an Open Source cross platform C,C++,PHP and Node.js IDE"
            VALUE "FileVersion",      "9.3.0"
            VALUE "InternalName",     "CodeLite"
            VALUE "LegalCopyright",   "Copyright (C) 2007-2016 Eran Ifrah"
            VALUE "LegalTrademarks",  "All rights reserved."
            VALUE "OriginalFilename", "codelite.exe"
            VALUE "PrivateBuild",     ""
            VALUE "ProductName",      "CodeLite"
            VALUE "ProductVersion",   "9.3.0"
            VALUE "SpecialBuild",     ""
            VALUE "Support",          "http://forums.codelite.org"
            VALUE "Users",            "Unlimited."
        } // BLOCK "040904E4"
    } // BLOCK "StringFileInfo"
    BLOCK "VarFileInfo"
    {
        VALUE "Translation", 0x409, 1252 // 1252 = 0x04E4
    } // BLOCK "VarFileInfo"
}
