<?php

/**
 * @method string getString()
 * @method void setInteger(integer $integer)
 * @method setString(integer $integer)
 */
class ClassWithMethods {
    public function real_func() {
    }
}

$c = new ClassWithMethods();
$c-> // 4 options


