//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// copyright            : (C) 2009 by Eran Ifrah
// file name            : svn_checkout_dialog.h
//
// -------------------------------------------------------------------------
// A
//              _____           _      _     _ _
//             /  __ \         | |    | |   (_) |
//             | /  \/ ___   __| | ___| |    _| |_ ___
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/
//              \____/\___/ \__,_|\___\_____/_|\__\___|
//
//                                                  F i l e
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#ifndef __svn_checkout_dialog__
#define __svn_checkout_dialog__

/**
@file
Subclass of SvnCheckoutDialogBase, which is generated by wxFormBuilder.
*/

#include "subversion2_ui.h"
class Subversion2;

/** Implementing SvnCheckoutDialogBase */
class SvnCheckoutDialog : public SvnCheckoutDialogBase
{
	Subversion2 *m_plugin;
protected:
	// Handlers for SvnCheckoutDialogBase events.
	void OnBrowseDirectory      ( wxCommandEvent  &event );
	void OnOkUI                 ( wxUpdateUIEvent &event );

	virtual void OnOK( wxCommandEvent& event );

public:
	/** Constructor */
	SvnCheckoutDialog( wxWindow* parent, Subversion2 *plugin );
	virtual ~SvnCheckoutDialog();

	wxString GetURL();
	wxString GetTargetDir();
};

#endif // __svn_checkout_dialog__
