//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// copyright            : (C) 2008 by Eran Ifrah                            
// file name            : movefuncimpldlg.h              
//                                                                          
// -------------------------------------------------------------------------
// A                                                                        
//              _____           _      _     _ _                            
//             /  __ \         | |    | |   (_) |                           
//             | /  \/ ___   __| | ___| |    _| |_ ___                      
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )                     
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/                     
//              \____/\___/ \__,_|\___\_____/_|\__\___|                     
//                                                                          
//                                                  F i l e                 
//                                                                          
//    This program is free software; you can redistribute it and/or modify  
//    it under the terms of the GNU General Public License as published by  
//    the Free Software Foundation; either version 2 of the License, or     
//    (at your option) any later version.                                   
//                                                                          
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
 #ifndef __movefuncimpldlg__
#define __movefuncimpldlg__

/**
@file
Subclass of MoveFuncImplBaseDlg, which is generated by wxFormBuilder.
*/

#include "movefuncimplbasedlg.h"

/** Implementing MoveFuncImplBaseDlg */
class MoveFuncImplDlg : public MoveFuncImplBaseDlg
{
	void OnButtonCancel(wxCommandEvent &e);
	void OnButtonOK(wxCommandEvent &e);
	
public:
	/** Constructor */
	MoveFuncImplDlg( wxWindow* parent, const wxString &text, const wxString &fileName );
	
	void SetText(const wxString &text);
	wxString GetText();
	
	void SetFileName(const wxString &fileName);
	wxString GetFileName();
};

#endif // __movefuncimpldlg__
