//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Copyright            : (C) 2015 Eran Ifrah
// File name            : quickdebugbase.h
//
// -------------------------------------------------------------------------
// A
//              _____           _      _     _ _
//             /  __ \         | |    | |   (_) |
//             | /  \/ ___   __| | ___| |    _| |_ ___
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/
//              \____/\___/ \__,_|\___\_____/_|\__\___|
//
//                                                  F i l e
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// This file was auto-generated by codelite's wxCrafter Plugin
// wxCrafter project file: quickdebug.wxcp
// Do not modify this file by hand!
//////////////////////////////////////////////////////////////////////

#ifndef CODELITE_FORMBUILDER_QUICKDEBUG_BASE_CLASSES_H
#define CODELITE_FORMBUILDER_QUICKDEBUG_BASE_CLASSES_H

#include <wx/settings.h>
#include <wx/xrc/xmlres.h>
#include <wx/xrc/xh_bmp.h>
#include <wx/dialog.h>
#include <wx/iconbndl.h>
#include <wx/artprov.h>
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/combobox.h>
#include <wx/arrstr.h>
#include <wx/button.h>
#include <wx/textctrl.h>
#include <wx/choice.h>
#if wxVERSION_NUMBER >= 2900
#include <wx/persist.h>
#include <wx/persist/toplevel.h>
#include <wx/persist/bookctrl.h>
#include <wx/persist/treebook.h>
#endif

class QuickDebugBase : public wxDialog
{
protected:
    wxStaticText* m_staticText2;
    wxComboBox* m_ExeFilepath;
    wxButton* m_buttonBrowseExe;
    wxStaticText* m_staticText5;
    wxComboBox* m_WD;
    wxButton* m_buttonBrowseWD;
    wxStaticText* m_staticText4;
    wxTextCtrl* m_textCtrlArgs;
    wxStaticText* m_staticText3;
    wxChoice* m_choiceDebuggers;
    wxStaticText* m_staticText12;
    wxTextCtrl* m_textCtrlDebuggerExec;
    wxButton* m_buttonBrowseDebugger;
    wxStaticText* m_staticText6;
    wxTextCtrl* m_textCtrlCmds;
    wxStdDialogButtonSizer* m_stdBtnSizer2;
    wxButton* m_buttonOK;
    wxButton* m_buttonCancel1;

protected:
    virtual void OnButtonBrowseExe(wxCommandEvent& event) { event.Skip(); }
    virtual void OnButtonBrowseWD(wxCommandEvent& event) { event.Skip(); }
    virtual void OnSelectAlternateDebugger(wxCommandEvent& event) { event.Skip(); }
    virtual void OnButtonDebug(wxCommandEvent& event) { event.Skip(); }
    virtual void OnButtonCancel(wxCommandEvent& event) { event.Skip(); }

public:
    wxStaticText* GetStaticText2() { return m_staticText2; }
    wxComboBox* GetExeFilepath() { return m_ExeFilepath; }
    wxButton* GetButtonBrowseExe() { return m_buttonBrowseExe; }
    wxStaticText* GetStaticText5() { return m_staticText5; }
    wxComboBox* GetWD() { return m_WD; }
    wxButton* GetButtonBrowseWD() { return m_buttonBrowseWD; }
    wxStaticText* GetStaticText4() { return m_staticText4; }
    wxTextCtrl* GetTextCtrlArgs() { return m_textCtrlArgs; }
    wxStaticText* GetStaticText3() { return m_staticText3; }
    wxChoice* GetChoiceDebuggers() { return m_choiceDebuggers; }
    wxStaticText* GetStaticText12() { return m_staticText12; }
    wxTextCtrl* GetTextCtrlDebuggerExec() { return m_textCtrlDebuggerExec; }
    wxButton* GetButtonBrowseDebugger() { return m_buttonBrowseDebugger; }
    wxStaticText* GetStaticText6() { return m_staticText6; }
    wxTextCtrl* GetTextCtrlCmds() { return m_textCtrlCmds; }
    QuickDebugBase(wxWindow* parent, wxWindowID id = wxID_ANY, const wxString& title = _("Quick Debug"), const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxSize(-1,-1), long style = wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER);
    virtual ~QuickDebugBase();
};

#endif
