//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// copyright            : (C) 2008 by Eran Ifrah
// file name            : editorsettingsmiscpanel.h
//
// -------------------------------------------------------------------------
// A
//              _____           _      _     _ _
//             /  __ \         | |    | |   (_) |
//             | /  \/ ___   __| | ___| |    _| |_ ___
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/
//              \____/\___/ \__,_|\___\_____/_|\__\___|
//
//                                                  F i l e
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#ifndef __editorsettingsmiscpanel__
#define __editorsettingsmiscpanel__

/**
@file
Subclass of EditorSettingsMiscBasePanel, which is generated by wxFormBuilder.
*/

#include "editorsettingsmiscbasepanel.h"
#include "treebooknodebase.h"
/** Implementing EditorSettingsMiscBasePanel */
class EditorSettingsMiscPanel
    : public EditorSettingsMiscBasePanel
    , public TreeBookNode<EditorSettingsMiscPanel>
{
    bool     m_restartRequired;
    bool     m_oldSetLocale;
    wxString m_oldpreferredLocale;
    bool     m_oldMswUseTheme;

protected:
    virtual void OnResetAnnoyingDialogsAnswers(wxCommandEvent& event);
    virtual void OnLogoutputCheckUpdateUI(wxUpdateUIEvent& event);
    int FindAvailableLocales();

    // Handlers for EditorSettingsMiscBasePanel events.
    void OnClearButtonClick( wxCommandEvent& event );
    void OnClearUI(wxUpdateUIEvent &e);
    void OnEnableThemeUI(wxUpdateUIEvent& event);
    void LocaleStaticUpdateUI(wxUpdateUIEvent& event);
    void LocaleChkUpdateUI(wxUpdateUIEvent& event);
    void LocaleChoiceUpdateUI(wxUpdateUIEvent& event);
    void OnLogVerbosityChanged(wxCommandEvent& event);
    void OnShowLogFile(wxCommandEvent& event);

public:
    /** Constructor */
    EditorSettingsMiscPanel( wxWindow* parent );
    void Save(OptionsConfigPtr options);
    virtual bool IsRestartRequired() {
        return this->m_restartRequired;
    }

};

#endif // __editorsettingsmiscpanel__
