#ifndef __DATABASE_ERROR_CODES_H__
#define __DATABASE_ERROR_CODES_H__

#define DATABASE_LAYER_OK 0
#define DATABASE_LAYER_ERROR 1
#define DATABASE_LAYER_INVALID_USER 2
#define DATABASE_LAYER_BAD_PASSWORD 3
#define DATABASE_LAYER_CONSTRAINT_VIOLATION 4
#define DATABASE_LAYER_SQL_SYNTAX_ERROR 5
#define DATABASE_LAYER_ALLOCATION_ERROR 6
#define DATABASE_LAYER_INCOMPATIBLE_FIELD_TYPE 7
#define DATABASE_LAYER_FIELD_NOT_IN_RESULTSET 8
#define DATABASE_LAYER_NO_ROWS_FOUND 9
#define DATABASE_LAYER_NON_UNIQUE_RESULTSET 10
#define DATABASE_LAYER_UNSUPPORTED_OPERATION 11
#define DATABASE_LAYER_ERROR_LOADING_LIBRARY 12

// Using 0 for now since this is replacing a
//  boolean for the return code and we don't want
//  to break existing code
#define DATABASE_LAYER_QUERY_RESULT_ERROR 0

#endif // __DATABASE_ERROR_CODES_H__
