<?php

// Start of Zend Utils v.

function zem_get_extension_info_by_id () {}

function zem_get_extension_info_by_name () {}

function zem_get_extensions_info () {}

function zem_get_license_info () {}

function zend_is_configuration_changed () {}

function zend_set_configuration_changed () {}

function zend_get_cfg_var () {}

function zend_restart_php () {}

function zend_get_file_size () {}

function zem_serial_number_info () {}

function zend_report_statistics () {}

function zend_disable_statistics () {}

function zend_deployment_library_path () {}

function zend_get_app_id_for_url () {}

function zend_get_app_id_for_path () {}

function zend_stats_type_to_string () {}

define ('SERIAL_NUMBER_ERROR', -1);
define ('SERIAL_NUMBER_INVALID', -2);
define ('ZEND_STATS_TYPE_OPLUS_UTILIZATION', 0);
define ('ZEND_STATS_TYPE_OPLUS_HITS', 1);
define ('ZEND_STATS_TYPE_OPLUS_MISSES', 2);
define ('ZEND_STATS_TYPE_OPLUS_FILES_CONSUMPTION', 3);
define ('ZEND_STATS_TYPE_OPLUS_MEMORY_CONSUMPTION', 4);
define ('ZEND_STATS_TYPE_OPLUS_MEMORY_WASTED', 5);
define ('ZEND_STATS_TYPE_DC_SHM_UTILITZATION', 6);
define ('ZEND_STATS_TYPE_DC_SHM_HITS', 7);
define ('ZEND_STATS_TYPE_DC_SHM_MISSES', 8);
define ('ZEND_STATS_TYPE_DC_DISK_HITS', 9);
define ('ZEND_STATS_TYPE_DC_DISK_MISSES', 10);
define ('ZEND_STATS_TYPE_DC_NUM_OF_NAMESPACES', 11);
define ('ZEND_STATS_TYPE_DC_SHM_NUM_OF_ENTRIES', 12);
define ('ZEND_STATS_TYPE_DC_DISK_NUM_OF_ENTRIES', 13);
define ('ZEND_STATS_TYPE_OUTPUT_SEND_TIME', 56);
define ('ZEND_STATS_TYPE_MOBILE_OUTPUT_SEND_TIME', 57);
define ('ZEND_STATS_TYPE_PC_NUM_OF_RULES', 14);
define ('ZEND_STATS_TYPE_PC_HITS', 15);
define ('ZEND_STATS_TYPE_PC_MISSES', 16);
define ('ZEND_STATS_TYPE_PC_AVG_PROC_TIME_NON_CACHED_PAGE', 18);
define ('ZEND_STATS_TYPE_PC_AVG_PROC_TIME_CACHED_PAGE', 19);
define ('ZEND_STATS_TYPE_PC_NON_HANDLED_REQUESTS', 17);
define ('ZEND_STATS_TYPE_JQ_JOBS_PER_STATUS', 20);
define ('ZEND_STATS_TYPE_JQ_JOBS_ENQUEUED', 21);
define ('ZEND_STATS_TYPE_JQ_JOBS_SCHEDULED_ENQUEUED', 22);
define ('ZEND_STATS_TYPE_JQ_JOBS_DEQUEUED', 23);
define ('ZEND_STATS_TYPE_MON_NUM_OF_EVENTS', 29);
define ('ZEND_STATS_TYPE_NUM_REQUESTS_PER_SECOND', 30);
define ('ZEND_STATS_TYPE_NUM_REQUESTS', 31);
define ('ZEND_STATS_TYPE_NUM_PHP_WORKERS', 32);
define ('ZEND_STATS_TYPE_AVG_REQUEST_PROCESSING_TIME', 33);
define ('ZEND_STATS_TYPE_MAX_REQUEST_PROCESSING_TIME', 34);
define ('ZEND_STATS_TYPE_MIN_REQUEST_PROCESSING_TIME', 35);
define ('ZEND_STATS_TYPE_AVG_REQUEST_PROCESSING_TIME_PER_APP', 36);
define ('ZEND_STATS_TYPE_AVG_MEMORY_USAGE', 37);
define ('ZEND_STATS_TYPE_AVG_CPU_USAGE', 38);
define ('ZEND_STATS_TYPE_AVG_REQUEST_OUTPUT_SIZE', 39);
define ('ZEND_STATS_TYPE_AVG_DATABASE_TIME', 40);
define ('ZEND_STATS_TYPE_AVG_NETWORK_TIME', 41);
define ('ZEND_STATS_TYPE_AVG_LOCAL_TIME', 42);
define ('ZEND_STATS_TYPE_ACTIVE_SESSIONS', 43);
define ('ZEND_STATS_TYPE_SC_AVG_SESSION_SIZE', 24);
define ('ZEND_STATS_TYPE_SC_MIN_SESSION_SIZE', 25);
define ('ZEND_STATS_TYPE_SC_MAX_SESSION_SIZE', 26);
define ('ZEND_STATS_TYPE_SC_SESSIONS_PER_APP', 27);
define ('ZEND_STATS_TYPE_SC_SESSIONS_DATA_SPACE', 28);
define ('ZEND_STATS_TYPE_MON_NUM_OF_INFO_EVENTS', 44);
define ('ZEND_STATS_TYPE_MON_NUM_OF_WARNING_EVENTS', 45);
define ('ZEND_STATS_TYPE_MON_NUM_OF_SEVERE_EVENTS', 46);
define ('ZEND_STATS_TYPE_BROWSERS_DISTRIBUTION', 47);
define ('ZEND_STATS_TYPE_OS_DISTRIBUTION', 48);
define ('ZEND_STATS_TYPE_GEO_DISTRIBUTION', 49);
define ('ZEND_STATS_TYPE_REQUESTS_LATENCY', 50);
define ('ZEND_STATS_TYPE_MOBILE_REQUESTS_LATENCY', 54);
define ('ZEND_STATS_TYPE_MOBILE_AVG_DATABASE_TIME', 51);
define ('ZEND_STATS_TYPE_MOBILE_AVG_NETWORK_TIME', 52);
define ('ZEND_STATS_TYPE_MOBILE_AVG_LOCAL_TIME', 53);
define ('ZEND_STATS_TYPE_AVG_MOBILE_REQUEST_PROCESSING_TIME', 55);

// End of Zend Utils v.
?>
