/*
    Name:       main.h
    Purpose:    Declares simple wxWidgets application with GUI
                created using wxCrafter.
    Author:    
    Created:   
    Copyright: 
    License:    wxWidgets license (www.wxwidgets.org)

    Notes:      Note that all GUI creation code is contained in
                the file gui.xrc which is generated by wxCrafter.
*/

#ifndef __MAIN_H__
#define __MAIN_H__

////////////////////////////////////////////////////////////////////////////////
// application class declaration 
////////////////////////////////////////////////////////////////////////////////

class MainApp : public wxApp
{
public:
    virtual bool OnInit();
};

// declare global static function wxGetApp()
DECLARE_APP(MainApp)

////////////////////////////////////////////////////////////////////////////////
// main application frame declaration 
////////////////////////////////////////////////////////////////////////////////

class MainFrame : public wxFrame
{
public:
    MainFrame(wxWindow *parent = NULL);
    ~MainFrame();
    
private:
    void OnCloseFrame(wxCloseEvent& event);
    void OnExitClick(wxCommandEvent& event);
};

#endif //__MAIN_H__
