#define CLANG_COMPLETION 257
#define CLANG_HIDDEN 258
#define CLANG_DELIM_OPEN 259
#define CLANG_DELIM_CLOSE 260
#define CLANG_ARG_DELIM_OPEN 261
#define CLANG_ARG_DELIM_CLOSE 262
#define CLANG_ARG_OPT_DELIM_OPEN 263
#define CLANG_ARG_OPT_DELIM_CLOSE 264
#define CLANG_STAR 265
#define CLANG_AMP 266
#define CLANG_TILDE 267
#define CLANG_ENUM 268
#define CLANG_ANONYMOUS 269
#define CLANG_CLCL 270
#define CLANG_OP_ASSIGN 271
#define CLANG_OP_DOTstar 272
#define CLANG_OP_ARROW 273
#define CLANG_OP_ARROWstar 274
#define CLANG_OP_ICR 275
#define CLANG_OP_DECR 276
#define CLANG_OP_LS 277
#define CLANG_OP_RS 278
#define CLANG_OP_LE 279
#define CLANG_OP_GE 280
#define CLANG_OP_EQ 281
#define CLANG_OP_NE 282
#define CLANG_ELLIPSIS 283
#define CLANG_NAME 284
#define CLANG_OPERATOR 285
