//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// copyright            : (C) 2014 Eran Ifrah
// file name            : newqtprojdlg.h
//
// -------------------------------------------------------------------------
// A
//              _____           _      _     _ _
//             /  __ \         | |    | |   (_) |
//             | /  \/ ___   __| | ___| |    _| |_ ___
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/
//              \____/\___/ \__,_|\___\_____/_|\__\___|
//
//                                                  F i l e
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#ifndef __newqtprojdlg__
#define __newqtprojdlg__

/**
@file
Subclass of NewQtProjBaseDlg, which is generated by wxFormBuilder.
*/

#include "NewQtProj.h"

class QmakeConf;
class IManager;

/** Implementing NewQtProjBaseDlg */
class NewQtProjDlg : public NewQtProjBaseDlg
{
    QmakeConf *m_conf;
    IManager  *m_mgr;

protected:
    virtual void OnOKUI(wxUpdateUIEvent& event);
    // Handlers for NewQtProjBaseDlg events.
    void OnNewQmakeSettings( wxCommandEvent& event );

public:
    /** Constructor */
    NewQtProjDlg( wxWindow* parent, QmakeConf *conf, IManager *mgr );
    virtual ~NewQtProjDlg();

    wxString GetProjectName    () const;
    wxString GetProjectPath    () const;
    wxString GetProjectKind    () const;
    wxString GetQmake          () const;
    bool     GetCreateDirectory() const;
};

#endif // __newqtprojdlg__
