//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Copyright            : (C) 2015 Eran Ifrah
// File name            : buildorderbasepage.h
//
// -------------------------------------------------------------------------
// A
//              _____           _      _     _ _
//             /  __ \         | |    | |   (_) |
//             | /  \/ ___   __| | ___| |    _| |_ ___
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/
//              \____/\___/ \__,_|\___\_____/_|\__\___|
//
//                                                  F i l e
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// This file was auto-generated by codelite's wxCrafter Plugin
// wxCrafter project file: depend_dlg_page.wxcp
// Do not modify this file by hand!
//////////////////////////////////////////////////////////////////////

#ifndef CODELITE_LITEEDITOR_DEPEND_DLG_PAGE_BASE_CLASSES_H
#define CODELITE_LITEEDITOR_DEPEND_DLG_PAGE_BASE_CLASSES_H

#include <wx/settings.h>
#include <wx/xrc/xmlres.h>
#include <wx/xrc/xh_bmp.h>
#include <wx/panel.h>
#include <wx/artprov.h>
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/choice.h>
#include <wx/arrstr.h>
#include <wx/checklst.h>
#include <wx/listbox.h>
#include <wx/button.h>
#if wxVERSION_NUMBER >= 2900
#include <wx/persist.h>
#include <wx/persist/toplevel.h>
#include <wx/persist/bookctrl.h>
#include <wx/persist/treebook.h>
#endif

class DependenciesPageBase : public wxPanel
{
protected:
    wxStaticText* m_staticText3;
    wxChoice* m_choiceProjectConfig;
    wxStaticText* m_staticText1;
    wxCheckListBox* m_checkListProjectList;
    wxStaticText* m_staticText2;
    wxListBox* m_listBoxBuildOrder;
    wxButton* m_buttonUp;
    wxButton* m_buttonDown;
    wxButton* m_buttonApply;

protected:
    virtual void OnConfigChanged(wxCommandEvent& event) { event.Skip(); }
    virtual void OnCheckListItemToggled(wxCommandEvent& event) { event.Skip(); }
    virtual void OnMoveUp(wxCommandEvent& event) { event.Skip(); }
    virtual void OnMoveDown(wxCommandEvent& event) { event.Skip(); }
    virtual void OnApplyButton(wxCommandEvent& event) { event.Skip(); }
    virtual void OnApplyButtonUI(wxUpdateUIEvent& event) { event.Skip(); }

public:
    wxStaticText* GetStaticText3() { return m_staticText3; }
    wxChoice* GetChoiceProjectConfig() { return m_choiceProjectConfig; }
    wxStaticText* GetStaticText1() { return m_staticText1; }
    wxCheckListBox* GetCheckListProjectList() { return m_checkListProjectList; }
    wxStaticText* GetStaticText2() { return m_staticText2; }
    wxListBox* GetListBoxBuildOrder() { return m_listBoxBuildOrder; }
    wxButton* GetButtonUp() { return m_buttonUp; }
    wxButton* GetButtonDown() { return m_buttonDown; }
    wxButton* GetButtonApply() { return m_buttonApply; }
    DependenciesPageBase(wxWindow* parent, wxWindowID id = wxID_ANY, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxSize(-1,-1), long style = wxTAB_TRAVERSAL);
    virtual ~DependenciesPageBase();
};

#endif
