<?php

// Start of Zend Monitor v.8.0.0

function monitor_pass_error () {}

function zend_monitor_pass_error () {}

function monitor_custom_event () {}

function zend_monitor_custom_event () {}

function monitor_set_aggregation_hint () {}

function zend_monitor_set_aggregation_hint () {}

function monitor_event_reporting () {}

function zend_monitor_event_reporting () {}

function monitor_report_event () {}

function zend_monitor_report_event () {}

function monitor_custom_event_ex () {}

function zend_monitor_custom_event_ex () {}

function monitor_collect_statistics () {}

function zend_monitor_collect_statistics () {}

define ('ZEND_MONITOR_ET_JQ_JOB_EXEC_ERROR', 8);
define ('ZEND_MONITOR_ET_JQ_JOB_LOGICAL_FAILURE', 9);
define ('ZEND_MONITOR_ET_JQ_JOB_EXEC_DELAY', 10);
define ('ZEND_MONITOR_ET_JQ_DAEMON_HIGH_CONCURRENCY_LEVEL', 11);
define ('ZEND_MONITOR_ET_TRACER_FILE_WRITE_FAIL', 12);
define ('ZEND_MONITOR_ET_TRACER_FILE_DUMP_TRUNCATED', 20);
define ('ZEND_MONITOR_ET_ZSM_CONFIGUATION_MISMATCH', 14);
define ('ZEND_MONITOR_ET_ZSM_NODE_ADDED_SUCCESSFULLY', 15);
define ('ZEND_MONITOR_ET_ZSM_NODE_IS_NOT_RESPONDING', 17);
define ('ZEND_MONITOR_ET_ZSM_NODE_REMOVED_SUCCESSFULLY', 16);
define ('ZEND_MONITOR_ET_ZSM_RESTART_FAILED', 13);
define ('ZEND_MONITOR_ET_ZSM_NODE_DISABLED', 18);
define ('ZEND_MONITOR_ET_ZSM_NODE_ENABLED', 19);
define ('ZEND_MONITOR_ET_DEPL_APP_DEPLOYED_ERROR', 22);
define ('ZEND_MONITOR_ET_DEPL_APP_DEPLOYED_SUCCESSFULLY', 21);
define ('ZEND_MONITOR_ET_DEPL_APP_REMOVED_ERROR', 26);
define ('ZEND_MONITOR_ET_DEPL_APP_REMOVED_SUCCESSFULLY', 25);
define ('ZEND_MONITOR_ET_DEPL_APP_RE_DEPLOYED_ERROR', 24);
define ('ZEND_MONITOR_ET_DEPL_APP_RE_DEPLOYED_SUCCESSFULLY', 23);
define ('ZEND_MONITOR_ET_DEPL_APP_UPDATE_ERROR', 28);
define ('ZEND_MONITOR_ET_DEPL_APP_UPDATE_SUCCESSFULLY', 27);
define ('ZEND_MONITOR_ET_DEPL_APP_ROLLBACKED_ERROR', 30);
define ('ZEND_MONITOR_ET_DEPL_APP_ROLLBACKED_SUCCESSFULLY', 29);
define ('ZEND_MONITOR_ETBM_CUSTOM', 1);
define ('ZEND_MONITOR_ETBM_FUNC_SLOW_EXEC', 2);
define ('ZEND_MONITOR_ETBM_FUNC_ERR', 4);
define ('ZEND_MONITOR_ETBM_REQ_SLOW_EXEC', 8);
define ('ZEND_MONITOR_ETBM_REQ_REL_SLOW_EXEC', 8);
define ('ZEND_MONITOR_ETBM_REQ_LARGE_MEM_USAGE', 16);
define ('ZEND_MONITOR_ETBM_REQ_REL_LARGE_MEM_USAGE', 16);
define ('ZEND_MONITOR_ETBM_REQ_LARGE_OUT_SIZE', 32);
define ('ZEND_MONITOR_ETBM_ZEND_ERR', 64);
define ('ZEND_MONITOR_ETBM_JAVA_EXP', 128);
define ('ZEND_MONITOR_ETBM_JQ_JOB_EXEC_ERROR', 256);
define ('ZEND_MONITOR_ETBM_JQ_JOB_LOGICAL_FAILURE', 512);
define ('ZEND_MONITOR_ETBM_JQ_JOB_EXEC_DELAY', 1024);
define ('ZEND_MONITOR_ETBM_JQ_DAEMON_HIGH_CONCURRENCY_LEVEL', 2048);
define ('ZEND_MONITOR_ETBM_TRACER_FILE_WRITE_FAIL', 4096);
define ('ZEND_MONITOR_ETBM_ALL', 2146697215);
define ('ZEND_MONITOR_ETBM_NONE', 1073741569);
define ('ZEND_MONITOR_ETBM_ZSM_CONFIGUATION_MISMATCH', 16384);
define ('ZEND_MONITOR_ETBM_ZSM_NODE_ADDED_SUCCESSFULLY', 32768);
define ('ZEND_MONITOR_ETBM_ZSM_NODE_DISABLED', 262144);
define ('ZEND_MONITOR_ETBM_ZSM_NODE_ENABLED', 524288);
define ('ZEND_MONITOR_ETBM_ZSM_NODE_IS_NOT_RESPONDING', 131072);
define ('ZEND_MONITOR_ETBM_ZSM_NODE_REMOVED_SUCCESSFULLY', 65536);
define ('ZEND_MONITOR_ETBM_ZSM_RESTART_FAILED', 8192);
define ('ZEND_MONITOR_ETBM_TRACER_FILE_DUMP_TRUNCATED', 1048576);
define ('ZEND_MONITOR_ETBM_DEPL_APP_DEPLOYED_ERROR', 4194304);
define ('ZEND_MONITOR_ETBM_DEPL_APP_DEPLOYED_SUCCESSFULLY', 2097152);
define ('ZEND_MONITOR_ETBM_DEPL_APP_REMOVED_ERROR', 67108864);
define ('ZEND_MONITOR_ETBM_DEPL_APP_REMOVED_SUCCESSFULLY', 33554432);
define ('ZEND_MONITOR_ETBM_DEPL_APP_RE_DEPLOYED_ERROR', 16777216);
define ('ZEND_MONITOR_ETBM_DEPL_APP_RE_DEPLOYED_SUCCESSFULLY', 8388608);
define ('ZEND_MONITOR_ETBM_DEPL_APP_UPDATE_ERROR', 268435456);
define ('ZEND_MONITOR_ETBM_DEPL_APP_UPDATE_SUCCESSFULLY', 134217728);
define ('ZEND_MONITOR_ETBM_DEPL_APP_ROLLBACKED_SUCCESSFULLY', 536870912);
define ('ZEND_MONITOR_ETBM_DEPL_APP_ROLLBACKED_ERROR', 1073741824);
define ('ZEND_MONITOR_EVENT_CUSTOM', 1);
define ('ZEND_MONITOR_EVENT_SLOWFUNC', 2);
define ('ZEND_MONITOR_EVENT_FUNCERROR', 4);
define ('ZEND_MONITOR_EVENT_SLOWSCRIPT', 8);
define ('ZEND_MONITOR_EVENT_SLOWSCRIPT_REL', 8);
define ('ZEND_MONITOR_EVENT_MEMUSAGE', 16);
define ('ZEND_MONITOR_EVENT_MEMUSAGE_REL', 16);
define ('ZEND_MONITOR_EVENT_OUTPUTSIZE', 32);
define ('ZEND_MONITOR_EVENT_PHPERROR', 64);
define ('ZEND_MONITOR_EVENT_JAVAEX', 128);
define ('ZEND_MONITOR_EVENT_JQ_JOB_EXEC_ERROR', 256);
define ('ZEND_MONITOR_EVENT_JQ_JOB_LOGICAL_FAILURE', 512);
define ('ZEND_MONITOR_EVENT_JQ_JOB_EXEC_DELAY', 1024);
define ('ZEND_MONITOR_EVENT_JQ_DAEMON_HIGH_CONCURRENCY_LEVEL', 2048);
define ('ZEND_MONITOR_EVENT_TRACER_FILE_WRITE_FAIL', 4096);
define ('ZEND_MONITOR_EVENT_TRACER_FILE_DUMP_TRUNCATED', 1048576);
define ('ZEND_MONITOR_EVENT_ALL', 2146697215);
define ('ZEND_MONITOR_EVENT_NONE', 1073741569);
define ('ZEND_MONITOR_EVENT_SEVERITY_INFO', -1);
define ('ZEND_MONITOR_EVENT_SEVERITY_WARNING', 0);
define ('ZEND_MONITOR_EVENT_SEVERITY_ERROR', 1);

// End of Zend Monitor v.8.0.0
?>
