//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Copyright            : (C) 2015 Eran Ifrah
// File name            : editorsettingsbookmarksbasepanel.h
//
// -------------------------------------------------------------------------
// A
//              _____           _      _     _ _
//             /  __ \         | |    | |   (_) |
//             | /  \/ ___   __| | ___| |    _| |_ ___
//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )
//             | \__/\ (_) | (_| |  __/ |___| | ||  __/
//              \____/\___/ \__,_|\___\_____/_|\__\___|
//
//                                                  F i l e
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// This file was auto-generated by codelite's wxCrafter Plugin
// wxCrafter project file: editor_options_bookmarks.wxcp
// Do not modify this file by hand!
//////////////////////////////////////////////////////////////////////

#ifndef CODELITE_LITEEDITOR_EDITOR_OPTIONS_BOOKMARKS_BASE_CLASSES_H
#define CODELITE_LITEEDITOR_EDITOR_OPTIONS_BOOKMARKS_BASE_CLASSES_H

#include <wx/settings.h>
#include <wx/xrc/xmlres.h>
#include <wx/xrc/xh_bmp.h>
#include <wx/panel.h>
#include <wx/artprov.h>
#include <wx/sizer.h>
#include <wx/checkbox.h>
#include <wx/stattext.h>
#include <wx/choice.h>
#include <wx/arrstr.h>
#include <wx/textctrl.h>
#include <wx/clrpicker.h>
#include <wx/statline.h>
#include <wx/spinctrl.h>
#if wxVERSION_NUMBER >= 2900
#include <wx/persist.h>
#include <wx/persist/toplevel.h>
#include <wx/persist/bookctrl.h>
#include <wx/persist/treebook.h>
#endif

class EditorSettingsBookmarksBasePanel : public wxPanel
{
protected:
    wxCheckBox* m_displaySelection;
    wxStaticText* m_staticText3;
    wxChoice* m_bookMarkShape;
    wxStaticText* m_staticText10;
    wxChoice* m_choiceBMType;
    wxStaticText* m_staticText41;
    wxTextCtrl* m_BookmarkLabel;
    wxStaticText* m_staticText4;
    wxColourPickerCtrl* m_backgroundColor;
    wxStaticText* m_staticText5;
    wxColourPickerCtrl* m_foregroundColor;
    wxStaticLine* m_staticLine18;
    wxStaticText* m_staticText66;
    wxColourPickerCtrl* m_highlightColor;
    wxStaticText* m_staticText54;
    wxSpinCtrl* m_spinCtrlHighlightAlpha;
    wxCheckBox* m_clearHighlitWords;

protected:
    virtual void OnBookmarkChanged(wxCommandEvent& event) { event.Skip(); }

public:
    wxCheckBox* GetDisplaySelection() { return m_displaySelection; }
    wxStaticText* GetStaticText3() { return m_staticText3; }
    wxChoice* GetBookMarkShape() { return m_bookMarkShape; }
    wxStaticText* GetStaticText10() { return m_staticText10; }
    wxChoice* GetChoiceBMType() { return m_choiceBMType; }
    wxStaticText* GetStaticText41() { return m_staticText41; }
    wxTextCtrl* GetBookmarkLabel() { return m_BookmarkLabel; }
    wxStaticText* GetStaticText4() { return m_staticText4; }
    wxColourPickerCtrl* GetBackgroundColor() { return m_backgroundColor; }
    wxStaticText* GetStaticText5() { return m_staticText5; }
    wxColourPickerCtrl* GetForegroundColor() { return m_foregroundColor; }
    wxStaticLine* GetStaticLine18() { return m_staticLine18; }
    wxStaticText* GetStaticText66() { return m_staticText66; }
    wxColourPickerCtrl* GetHighlightColor() { return m_highlightColor; }
    wxStaticText* GetStaticText54() { return m_staticText54; }
    wxSpinCtrl* GetSpinCtrlHighlightAlpha() { return m_spinCtrlHighlightAlpha; }
    wxCheckBox* GetClearHighlitWords() { return m_clearHighlitWords; }
    EditorSettingsBookmarksBasePanel(wxWindow* parent, wxWindowID id = wxID_ANY, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxSize(-1,-1), long style = wxTAB_TRAVERSAL);
    virtual ~EditorSettingsBookmarksBasePanel();
};

#endif
